@@ -108,7 +108,7 @@ QUimInputContext::QUimInputContext( const char *imname )
     mTextUtil = new QUimTextUtil( this );
 
     // read configuration
-    readIMConf();
+    updatePosition();
 
     m_indicator = new CaretStateIndicator;
 }
@@ -176,6 +176,11 @@ uim_context QUimInputContext::createUimContext( const char *imname )
                                  QUimTextUtil::acquire_text_cb,
                                  QUimTextUtil::delete_text_cb);
 
+#if UIM_QT_USE_DELAY
+    uim_set_delay_candidate_selector_cb( uc,
+        QUimInputContext::cand_activate_with_delay_cb );
+#endif /* !UIM_QT_USE_DELAY */
+
     uim_prop_list_update( uc );
 
     return uc;
@@ -183,12 +188,29 @@ uim_context QUimInputContext::createUimContext( const char *imname )
 
 void QUimInputContext::createCandidateWindow()
 {
+    cwin = 0;
+    // uim-candwin-prog is deprecated
     char *candwinprog = uim_scm_symbol_value_str( "uim-candwin-prog" );
-    if ( candwinprog && !strncmp( candwinprog, "uim-candwin-tbl", 15 ) )
-        cwin = new CandidateTableWindow( 0 );
-    else
-        cwin = new CandidateWindow( 0 );
+    if ( candwinprog ) {
+        if ( !strncmp( candwinprog, "uim-candwin-tbl", 15 ) )
+            cwin = new CandidateTableWindow( 0 );
+        else if ( !strncmp( candwinprog, "uim-candwin-horizontal", 22 ) )
+            cwin = new CandidateWindow( 0, false );
+    } else {
+        char *style = uim_scm_symbol_value_str( "candidate-window-style" );
+        if ( style ) {
+            if ( !strcmp( style, "table" ) )
+                cwin = new CandidateTableWindow( 0 );
+            else if ( !strcmp( style, "horizontal" ) )
+                cwin = new CandidateWindow( 0, false );
+        }
+        free( style );
+    }
     free( candwinprog );
+    
+    if ( !cwin )
+        cwin = new CandidateWindow( 0 );
+
     cwin->setQUimInputContext( this );
     cwin->hide();
 }
@@ -567,7 +589,6 @@ void QUimInputContext::cand_activate_cb( void *ptr, int nr, int displayLimit )
 
     QUimInputContext *ic = static_cast<QUimInputContext*>( ptr );
     ic->cwin->candidateActivate( nr, displayLimit );
-    ic->candwinIsActive = true;
 }
 
 void QUimInputContext::cand_select_cb( void *ptr, int index )
@@ -613,6 +634,14 @@ void QUimInputContext::switch_system_global_im_cb( void *ptr, const char *name )
     ic->switch_system_global_im( name );
 }
 
+#if UIM_QT_USE_DELAY
+void QUimInputContext::cand_activate_with_delay_cb( void *ptr, int delay )
+{
+    QUimInputContext *ic = static_cast<QUimInputContext*>( ptr );
+    ic->cwin->candidateActivateWithDelay( delay );
+}
+#endif /* !UIM_QT_USE_DELAY */
+
 void QUimInputContext::commitString( const QString& str )
 {
     QInputMethodEvent e;
@@ -674,12 +703,28 @@ void QUimInputContext::savePreedit()
 
 void QUimInputContext::restorePreedit()
 {
+    AbstractCandidateWindow *window = cwinHash.take( focusedWidget );
+    // if window is 0, updateStyle() was called.
+    if ( !window ) {
+        psegs = psegsHash.take( focusedWidget );
+        QString preedit;
+        while ( !psegs.isEmpty() ) {
+            preedit += psegs.takeFirst().str;
+        }
+        commitString( preedit );
+
+        uim_context uc = m_ucHash.take( focusedWidget );
+        if ( uc )
+            uim_release_context( uc );
+        visibleHash.remove( focusedWidget );
+        return;
+    }
     if ( m_uc )
         uim_release_context( m_uc );
     delete cwin;
     m_uc = m_ucHash.take( focusedWidget );
     psegs = psegsHash.take( focusedWidget );
-    cwin = cwinHash.take( focusedWidget );
+    cwin = window;
     if ( visibleHash.take( focusedWidget ) )
         cwin->popup();
 }
@@ -687,7 +732,7 @@ void QUimInputContext::restorePreedit()
 
 void QUimInputContext::saveContext()
 {
-    // just send IMEnd and keep preedit string
+    // just send QInputMethodEvent and keep preedit string
     if ( isComposing() )
         commitString( "" );
 }
@@ -873,7 +918,7 @@ void QUimInputContext::switch_app_global_im( const char *name )
     {
         if ( ( *it ) != this) {
             uim_switch_im( ( *it )->uimContext(), name );
-            ( *it )->readIMConf();
+            ( *it )->updatePosition();
         }
     }
     uim_prop_update_custom(this->uimContext(), "custom-preserved-default-im-name", im_name_sym.toUtf8().data() );
@@ -885,13 +930,35 @@ void QUimInputContext::switch_system_global_im( const char *name )
     QUimHelperManager::send_im_change_whole_desktop( name );
 }
 
-void QUimInputContext::readIMConf()
+void QUimInputContext::updatePosition()
 {
     char * leftp = uim_scm_symbol_value_str( "candidate-window-position" );
     cwin->setAlwaysLeftPosition( leftp && !strcmp( leftp, "left" ) );
     free( leftp );
 }
 
+void QUimInputContext::updateStyle()
+{
+    // don't update window style if deprecated uim-candwin-prog is set
+    char *candwinprog = uim_scm_symbol_value_str( "uim-candwin-prog" );
+    if ( candwinprog ) {
+        free( candwinprog );
+        return;
+    }
+    delete cwin;
+    createCandidateWindow();
+#ifdef WORKAROUND_BROKEN_RESET_IN_QT4
+    // invalidate all the candidate windows stored in cwinHash
+    QHashIterator<QWidget*, AbstractCandidateWindow*> i( cwinHash );
+    while ( i.hasNext() ) {
+        i.next();
+        QWidget *widget = i.key();
+        delete cwinHash[ widget ];
+        cwinHash[ widget ] = 0;
+    }
+#endif
+}
+
 void QUimInputContext::updateIndicator( const QString &str )
 {
     m_indicator->update( str );
