@@ -95,7 +95,8 @@ public:
 
     void commitString( const QString& str );
 
-    void readIMConf();
+    void updatePosition();
+    void updateStyle();
 
     QUimTextUtil *textUtil() { return mTextUtil; }
 
@@ -107,6 +108,8 @@ public:
 
     void updateIndicator( const QString &str );
 
+    void setCandwinActive() { candwinIsActive = true; }
+
 protected:
     uim_context createUimContext( const char *imname );
     void createCandidateWindow();
@@ -136,6 +139,8 @@ private:
     //imsw
     static void switch_app_global_im_cb( void *ptr, const char *str );
     static void switch_system_global_im_cb( void *ptr, const char *str );
+    //delay
+    static void cand_activate_with_delay_cb( void *ptr, int delay );
     /* real functions for callbacks (correspond order) */
     //preedit
     void clearPreedit();
