@@ -874,8 +874,11 @@ void QUimInputContext::create_compose_tree()
         }
     }
 
-    if (name[0] == '\0' && !get_compose_filename(name, sizeof(name)))
+    if (name[0] == '\0' && !get_compose_filename(name, sizeof(name))) {
+        if (fp)
+            fclose(fp);
         return;
+    }
 
     if (fp == 0 && ((fp = fopen(name, "r")) == 0))
         return;
