@@ -39,7 +39,10 @@
 
 #include <QtCore/QPointer>
 #include <QtGui/QFileDialog>
+#include <QtGui/QHeaderView>
 #include <QtGui/QLabel>
+#include <QtGui/QPushButton>
+#include <QtGui/QTableWidget>
 #include <QtGui/QTreeWidget>
 #include <QtGui/QTreeWidgetItem>
 #include <QtGui/QVBoxLayout>
@@ -237,7 +240,7 @@ void CustomPathnameEdit::setDefault()
 
 void CustomPathnameEdit::slotPathnameButtonClicked()
 {
-    m_fileDialog = new QFileDialog( this, "file dialog" );
+    m_fileDialog = new QFileDialog( this, _("Specify file") );
 
     switch (m_custom->value->as_pathname->type) {
     case UCustomPathnameType_Directory:
@@ -362,7 +365,7 @@ CustomOrderedListEdit::CustomOrderedListEdit( struct uim_custom *c, QWidget *par
     m_lineEdit->setReadOnly( true );
 
     m_editButton = new QPushButton( this );
-    m_editButton->setText( _("Edit") );
+    m_editButton->setText( _("Edit...") );
     connect( m_editButton, SIGNAL(clicked()),
                       this, SLOT(slotEditButtonClicked()) );
 
@@ -462,6 +465,7 @@ void CustomOrderedListEdit::initPtrList()
 void CustomOrderedListEdit::slotEditButtonClicked()
 {
     OListEditForm *d = new OListEditForm( this );
+    d->setWindowTitle( _FU8( m_custom->label ) );
     initPtrList();
 
     /*
@@ -568,10 +572,8 @@ void CustomOrderedListEdit::updateText()
     m_lineEdit->setText( str );
 }
 
-OListEditForm::OListEditForm( QWidget *parent )
-    : QDialog( parent )
+OListEditForm::OListEditForm( QWidget *parent ) : OListEditFormBase( parent )
 {
-    setupUi( this );
     m_listView->setRootIsDecorated( false );
     connect( m_upButton, SIGNAL(clicked()),
                       this, SLOT(upItem()) );
@@ -642,7 +644,7 @@ CustomKeyEdit::CustomKeyEdit( struct uim_custom *c, QWidget *parent )
     m_lineEdit->setReadOnly( true );
 
     m_editButton = new QPushButton( this );
-    m_editButton->setText( _("Edit") );
+    m_editButton->setText( _("Edit...") );
     connect( m_editButton, SIGNAL(clicked()),
                       this, SLOT(slotKeyButtonClicked()) );
 
@@ -734,6 +736,8 @@ void CustomKeyEdit::setDefault()
 void CustomKeyEdit::slotKeyButtonClicked()
 {
     KeyEditForm *d = new KeyEditForm( this );
+    d->setWindowTitle(
+            _( "%1 - key configuration" ).arg( _FU8( m_custom->label ) ) );
 
     /* add items */
     QString str;
@@ -793,11 +797,8 @@ void CustomKeyEdit::slotKeyButtonClicked()
     delete d;
 }
 
-KeyEditForm::KeyEditForm( QWidget *parent )
-    : QDialog( parent )
+KeyEditForm::KeyEditForm( QWidget *parent ) : KeyEditFormBase( parent )
 {
-    setupUi( this );
-
     m_listView->setRootIsDecorated( false );
     m_removeButton->setEnabled( false );
     m_editButton->setEnabled( false );
@@ -1200,6 +1201,294 @@ void KeyGrabDialog::setKeyStr()
         
 }
 
+//----------------------------------------------------------------------------------------
+CustomTable::CustomTable( struct uim_custom *c, QWidget *parent )
+    : QFrame( parent ),
+      UimCustomItemIface( c )
+{
+    QPushButton *editButton = new QPushButton;
+    editButton->setText( _("Edit...") );
+    connect( editButton, SIGNAL(clicked()),
+            this, SLOT(slotEditButtonClicked()) );
+
+    QHBoxLayout *layout = new QHBoxLayout;
+    layout->setMargin( 0 );
+    layout->setSpacing( 3 );
+    layout->addStretch();
+    layout->addWidget( editButton );
+
+    setLayout( layout );
+
+    update();
+}
+
+void CustomTable::update()
+{
+    if( !m_custom || m_custom->type != UCustom_Table )
+        return;
+    /* sync with Label */
+    parentWidget()->setEnabled( m_custom->is_active );
+}
+
+void CustomTable::setDefault()
+{
+    char ***custom_table = m_custom->value->as_table;
+    for ( int row = 0; custom_table[row]; row++ ) {
+        for ( int column = 0; custom_table[row][column]; column++ ) {
+            free( custom_table[row][column] );
+        }
+        free( custom_table[row] );
+    }
+    char ***default_table = m_custom->default_value->as_table;
+    int row;
+    for ( row = 0; default_table[row]; row++ )
+        ;
+    custom_table = (char ***)malloc( sizeof(char **) * ( row + 1 ) );
+    custom_table[row] = 0;
+
+    m_custom->value->as_table = custom_table;
+
+    for ( int row = 0; default_table[row]; row++ ) {
+        int column;
+        // the number of column may differ from row to row
+        for ( column = 0; default_table[row][column]; column++ )
+            ;
+        custom_table[row] = (char **)malloc( sizeof(char *) * ( column + 1 ) );
+        custom_table[row][column] = 0;
+        for ( int column = 0; default_table[row][column]; column++ )
+            custom_table[row][column] = strdup( default_table[row][column] );
+    }
+
+    setCustom( m_custom );
+    update();
+}
+
+void CustomTable::slotEditButtonClicked()
+{
+    QPointer<TableEditForm> d = new TableEditForm( this );
+    d->setWindowTitle( _FU8( m_custom->label ) );
+    d->setTable( m_custom->value->as_table );
+    int column = 0;
+    for ( struct uim_custom_choice **item
+            = m_custom->range->as_table_header.valid_items;
+            *item; item++ ) {
+        d->setTableHeaderItem( _FU8( ( *item )->label ), column );
+        column++;
+    }
+    if ( d->exec() == QDialog::Accepted ) {
+        m_custom->value->as_table = d->table();
+        setCustom( m_custom );
+        update();
+    }
+    delete d;
+}
+
+TableEditForm::TableEditForm( QWidget *parent )
+    : QDialog( parent )
+{
+    m_table = new QTableWidget;
+    m_table->setSelectionMode( QAbstractItemView::SingleSelection );
+    QHeaderView *verticalHeader = m_table->verticalHeader();
+    verticalHeader->setVisible( false );
+    verticalHeader->setDefaultSectionSize( 
+            QFontMetrics( m_table->font() ).height() + 2 );
+    QHeaderView *horizontalHeader = m_table->horizontalHeader();
+    horizontalHeader->setResizeMode( QHeaderView::ResizeToContents );
+    horizontalHeader->setStretchLastSection( true );
+    connect( m_table, SIGNAL( itemSelectionChanged() ),
+            this, SLOT( slotItemSelectionChanged() ) );
+
+    QPushButton *addButton = new QPushButton;
+    addButton->setText( _("Add") );
+    connect( addButton, SIGNAL( clicked() ),
+            this, SLOT( slotAddClicked() ) );
+
+    m_removeButton = new QPushButton;
+    m_removeButton->setEnabled( false );
+    m_removeButton->setText( _("Remove") );
+    connect( m_removeButton, SIGNAL( clicked() ),
+            this, SLOT( slotRemoveClicked() ) );
+
+    m_upButton = new QPushButton;
+    m_upButton->setEnabled( false );
+    m_upButton->setText( _("Up") );
+    connect( m_upButton, SIGNAL( clicked() ),
+            this, SLOT( slotUpClicked() ) );
+
+    m_downButton = new QPushButton;
+    m_downButton->setEnabled( false );
+    m_downButton->setText( _("Down") );
+    connect( m_downButton, SIGNAL( clicked() ),
+            this, SLOT( slotDownClicked() ) );
+
+    QPushButton *okButton = new QPushButton;
+    okButton->setText( _("OK") );
+    connect( okButton, SIGNAL( clicked() ), this, SLOT( accept() ) );
+
+    QPushButton *cancelButton = new QPushButton;
+    cancelButton->setText( _("Cancel") );
+    connect( cancelButton, SIGNAL( clicked() ), this, SLOT( reject() ) );
+
+    QVBoxLayout *buttonLayout = new QVBoxLayout;
+    buttonLayout->addWidget( addButton );
+    buttonLayout->addWidget( m_removeButton );
+    buttonLayout->addWidget( m_upButton );
+    buttonLayout->addWidget( m_downButton );
+    buttonLayout->addStretch();
+    buttonLayout->addWidget( okButton );
+    buttonLayout->addWidget( cancelButton );
+
+    QHBoxLayout *layout = new QHBoxLayout;
+    layout->addWidget( m_table );
+    layout->addLayout( buttonLayout );
+
+    setLayout( layout );
+
+    m_table->horizontalHeader()->adjustSize();
+}
+
+void TableEditForm::setTable( char ***custom_table )
+{
+    if ( !custom_table )
+        return;
+    // the number of column may differ from row to row
+    int max_column = -1;
+    int row;
+    for ( row = 0; custom_table[row]; row++ ) {
+        for ( int column = 0; custom_table[row][column]; column++ ) {
+            if ( max_column < column )
+                max_column = column;
+        }
+    }
+    m_table->setRowCount( row );
+    m_table->setColumnCount( max_column + 1 );
+
+    for ( int row = 0; custom_table[row]; row++ ) {
+        bool expanded = false;
+        for ( int column = 0; column < max_column + 1; column++ ) {
+            if ( !custom_table[row][column] )
+                expanded = true;
+            QTableWidgetItem *item = new QTableWidgetItem( expanded ?
+                "" : _FU8( custom_table[row][column] ) );
+            if ( expanded )
+                item->setFlags( Qt::NoItemFlags );
+            m_table->setItem( row, column, item );
+        }
+    }
+    m_customTable = custom_table;
+}
+
+char ***TableEditForm::table() const
+{
+    char ***custom_table = m_customTable;
+    for ( int row = 0; custom_table[row]; row++ ) {
+        for ( int column = 0; custom_table[row][column]; column++ ) {
+            free( custom_table[row][column] );
+        }
+        free( custom_table[row] );
+    }
+
+    int rowCount = m_table->rowCount();
+    custom_table = (char ***)malloc( sizeof(char **) * ( rowCount + 1 ) );
+    custom_table[rowCount] = 0;
+
+    int columnCount = m_table->columnCount();
+
+    for ( int row = 0; row < rowCount; row++ ) {
+        int columnCountForRow = columnCount;
+        // the number of column may differ from row to row
+        for ( int column = 0; column < columnCount; column++ ) {
+            if ( !( m_table->item( row, column )->flags() ) ) {
+                columnCountForRow = column;
+                break;
+            }
+        }
+        custom_table[row]
+            = (char **)malloc( sizeof(char *) * ( columnCountForRow + 1 ) );
+        custom_table[row][columnCountForRow] = 0;
+        for ( int column = 0; column < columnCountForRow; column++ )
+            custom_table[row][column] = strdup(
+                m_table->item( row, column )->text().toUtf8().data() );
+    }
+
+    return custom_table;
+}
+
+void TableEditForm::setTableHeaderItem( const QString &item, int column )
+{
+    if ( column < m_table->columnCount() ) {
+        m_table->setHorizontalHeaderItem( column,
+                new QTableWidgetItem( item ) );
+    }
+}
+
+void TableEditForm::slotItemSelectionChanged()
+{
+    bool itemSelected = ( m_table->selectedItems().count() == 1 );
+    m_removeButton->setEnabled( itemSelected );
+    m_upButton->setEnabled( itemSelected );
+    m_downButton->setEnabled( itemSelected );
+}
+
+void TableEditForm::slotAddClicked()
+{
+    QList<QTableWidgetItem *> items = m_table->selectedItems();
+    int row = ( items.count() > 0 ) ? items[0]->row() + 1 : m_table->rowCount();
+    m_table->insertRow( row );
+    for ( int i = 0; i < m_table->columnCount(); i++ )
+        m_table->setItem( row, i, new QTableWidgetItem( "" ) );
+    m_table->scrollToItem( m_table->item( row, 0 ) );
+}
+
+void TableEditForm::slotRemoveClicked()
+{
+    QList<QTableWidgetItem *> items = m_table->selectedItems();
+    if ( items.count() != 1 )
+        return;
+    m_table->removeRow( items[0]->row() );
+}
+
+void TableEditForm::slotUpClicked()
+{
+    QList<QTableWidgetItem *> items = m_table->selectedItems();
+    if ( items.count() != 1 )
+        return;
+    QTableWidgetItem *item = items[0];
+    const int row = item->row();
+    if ( row < 1 || row > m_table->rowCount() - 1 )
+        return;
+    const int newRow = row - 1;
+    m_table->insertRow( newRow );
+    for ( int column = 0; column < m_table->columnCount(); column++ ) {
+        m_table->setItem( newRow, column,
+                m_table->takeItem( row + 1, column ) );
+    }
+    m_table->removeRow( row + 1 );
+
+    m_table->setCurrentItem( item );
+    m_table->scrollToItem( item );
+}
+
+void TableEditForm::slotDownClicked()
+{
+    QList<QTableWidgetItem *> items = m_table->selectedItems();
+    if ( items.count() != 1 )
+        return;
+    QTableWidgetItem *item = items[0];
+    const int row = item->row();
+    if ( row < 0 || row >= m_table->rowCount() - 1 )
+        return;
+    const int newRow = row + 2;
+    m_table->insertRow( newRow );
+    for ( int column = 0; column < m_table->columnCount(); column++ ) {
+        m_table->setItem( newRow, column, m_table->takeItem( row, column ) );
+    }
+    m_table->removeRow( row );
+
+    m_table->setCurrentItem( item );
+    m_table->scrollToItem( item );
+}
+
 static QString unicodeKeyToSymStr ( QChar c )
 {
     QString str;
