@@ -52,6 +52,19 @@
 #endif
 #include <limits.h>
 
+#include <sys/param.h>
+#ifndef MAXPATHLEN
+# ifdef PATH_MAX
+#  define MAXPATHLEN PATH_MAX
+# else /* PATH_MAX */
+#  define MAXPATHLEN 1024	/* 64 in openssh-portable */
+# endif /* PATH_MAX */
+#endif /* MAXPATHLEN */
+
+#ifndef PATH_MAX
+# define PATH_MAX 1024		/* _POSIX_PATH_MAX in openssh-portable */
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
