@@ -1,7 +1,7 @@
 ;;; anthy.scm: Anthy for uim.
 ;;; charset: EUC-JP
 ;;;
-;;; Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2012 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -30,11 +30,11 @@
 ;;; SUCH DAMAGE.
 ;;;;
 
+(require-extension (srfi 1 8))
+
 (require "util.scm")
 (require "ustr.scm")
 (require "japanese.scm")
-(require "japanese-kana.scm")
-(require "japanese-azik.scm")
 (require-custom "generic-key-custom.scm")
 (require-custom "anthy-custom.scm")
 (require-custom "anthy-key-custom.scm")
@@ -55,6 +55,8 @@
 (define anthy-input-rule-roma 0)
 (define anthy-input-rule-kana 1)
 (define anthy-input-rule-azik 2)
+(define anthy-input-rule-act 3)
+(define anthy-input-rule-kzik 4)
 
 (define anthy-candidate-type-katakana -2)
 (define anthy-candidate-type-hiragana -3)
@@ -68,6 +70,15 @@
 ;; I don't think the key needs to be customizable.
 (define-key anthy-space-key? '(" "))
 
+;; Handle Anthy's version scheme like 7100b, 8158memm.
+(define anthy-version->major.minor
+  (lambda (vstr)
+    (if (string=? vstr "(unknown)")
+	'("-1" . "")
+	(receive (maj min) (span char-numeric? (string->list vstr))
+	  (cons (list->string maj)
+		(list->string min))))))
+
 (define anthy-prepare-input-rule-activation
   (lambda (ac)
     (cond
@@ -165,7 +176,7 @@
 		   '(ja_halfwidth_alnum
 		     "a"
 		     "Ⱦѱѿ"
-		     "Ⱦѱѿ⡼"))
+		     "Ⱦѱѿϥ⡼"))
 		 (lambda (ac)
 		   (and (anthy-context-on ac)
 			(anthy-context-alnum ac)
@@ -245,6 +256,7 @@
 		      anthy-input-rule-kana))
 		 (lambda (ac)
 		   (anthy-prepare-input-rule-activation ac)
+                   (require "japanese-kana.scm")
 		   (anthy-context-set-input-rule! ac anthy-input-rule-kana)
 		   (anthy-context-change-kana-mode! ac (anthy-context-kana-mode ac))
 		   (anthy-context-set-alnum! ac #f)
@@ -267,11 +279,49 @@
 		      anthy-input-rule-azik))
 		 (lambda (ac)
 		   (anthy-prepare-input-rule-activation ac)
+                   (require "japanese-azik.scm")
 		   (rk-context-set-rule! (anthy-context-rkc ac)
 					 ja-azik-rule)
 		   (japanese-roma-set-yen-representation)
 		   (anthy-context-set-input-rule! ac anthy-input-rule-azik)))
 
+(register-action 'action_anthy_kzik
+;;		 (indication-alist-indicator 'action_anthy_kzik
+;;					     anthy-kana-input-method-indication-alist)
+		 (lambda (ac)
+		   '(ja_kzik
+		     ""
+		     "KZIK"
+		     "KZIKĥ޻ϥ⡼"))
+		 (lambda (ac)
+		   (= (anthy-context-input-rule ac)
+		      anthy-input-rule-kzik))
+		 (lambda (ac)
+		   (anthy-prepare-input-rule-activation ac)
+                   (require "japanese-kzik.scm")
+		   (rk-context-set-rule! (anthy-context-rkc ac)
+					 ja-kzik-rule)
+		   (japanese-roma-set-yen-representation)
+		   (anthy-context-set-input-rule! ac anthy-input-rule-kzik)))
+
+(register-action 'action_anthy_act
+		 (lambda (ac)
+		   '(ja_act
+		     ""
+		     "ACT"
+		     "ACTĥ޻ϥ⡼"))
+		 (lambda (ac)
+		   (= (anthy-context-input-rule ac)
+		      anthy-input-rule-act))
+		 (lambda (ac)
+		   (anthy-prepare-input-rule-activation ac)
+                   (require "japanese-act.scm")
+		   (rk-context-set-rule! (anthy-context-rkc ac)
+					 ja-act-rule)
+		   (japanese-roma-set-yen-representation)
+		   (anthy-context-set-input-rule! ac anthy-input-rule-act)))
+
+
 ;; Update widget definitions based on action configurations. The
 ;; procedure is needed for on-the-fly reconfiguration involving the
 ;; custom API
@@ -319,23 +369,15 @@
      (if (symbol-bound? 'anthy-lib-init)
          (begin
 	   (set! anthy-lib-initialized? (anthy-lib-init))
-	   (set! anthy-version (anthy-lib-get-anthy-version))))
+	   (set! anthy-version (anthy-version->major.minor
+				(anthy-lib-get-anthy-version)))))
      (if anthy-lib-initialized?
 	 (anthy-context-set-ac-id! ac (anthy-lib-alloc-context)))
      (anthy-context-set-widgets! ac anthy-widgets)
      (anthy-context-set-rkc! ac rkc)
-     (anthy-context-set-preconv-ustr! ac (ustr-new))
-     (anthy-context-set-raw-ustr! ac (ustr-new))
-     (anthy-context-set-segments! ac (ustr-new))
-
-     ;; 2004-08-26 Takuro Ashie <ashie@homa.ne.jp>
-     ;;   * I think load-kana-table should be marked as depracated.
-     ;;     Because it is a little violent (it overwrites ja-rk-rule table).
-     ;;     We should prepare a custom entry like "uim-default-input-rule"
-     ;;     instead of using-kana-table.
-     (if using-kana-table?
-	 (anthy-context-set-input-rule! ac anthy-input-rule-kana)
-	 (anthy-context-set-input-rule! ac anthy-input-rule-roma))
+     (anthy-context-set-preconv-ustr! ac (ustr-new '()))
+     (anthy-context-set-raw-ustr! ac (ustr-new '()))
+     (anthy-context-set-segments! ac (ustr-new '()))
      ac)))
 
 (define anthy-commit-raw
@@ -388,8 +430,10 @@
 	      (string-append-map-ustr-former extract-kana preconv-str)
 	      (if convert-pending-into-kana?
 		  (if residual-kana
-		      (extract-kana residual-kana)
-		      pending)
+                    (if (list? (car residual-kana))
+                      (string-append-map extract-kana residual-kana)
+		      (extract-kana residual-kana))
+                    pending)
 		  pending)
 	      (string-append-map-ustr-latter extract-kana preconv-str))))
 	   kana)
@@ -397,8 +441,10 @@
 	   (string-append-map-ustr-former extract-kana preconv-str)
 	   (if convert-pending-into-kana?
 	       (if residual-kana
-		   (extract-kana residual-kana)
-		   "")
+                 (if (list? (car residual-kana))
+                   (string-append-map extract-kana residual-kana)
+		   (extract-kana residual-kana))
+                 "")
 	       pending)
 	   (string-append-map-ustr-latter extract-kana preconv-str))))))
 
@@ -411,8 +457,8 @@
 	      (if upper?
 		  (map
 		   (lambda (x)
-		     (if (char-alphabetic? (string->charcode x))
-			 (charcode->string (char-upcase (string->charcode x)))
+		     (if (ichar-alphabetic? (string->charcode x))
+			 (charcode->string (ichar-upcase (string->charcode x)))
 			 x))
 		   (string-to-list (car raw-str-list)))
 		  (string-to-list (car raw-str-list))))
@@ -422,8 +468,8 @@
 		 (string-list-concat
 		  (map
 		   (lambda (x)
-		     (if (char-alphabetic? (string->charcode x))
-			 (charcode->string (char-upcase (string->charcode x)))
+		     (if (ichar-alphabetic? (string->charcode x))
+			 (charcode->string (ichar-upcase (string->charcode x)))
 			 x))
 		   (string-to-list (car raw-str-list))))
 		 (car raw-str-list))
@@ -441,7 +487,7 @@
 (define anthy-release-handler
   (lambda (ac)
     (let ((ac-id (anthy-context-ac-id ac)))
-      (if (number? ac-id)
+      (if ac-id
 	  (anthy-lib-free-context ac-id)))))
 
 (define anthy-flush
@@ -532,7 +578,7 @@
     (let* ((ac-id (anthy-context-ac-id ac))
 	   (kana (anthy-context-kana-mode ac))
 	   (preconv-str (anthy-make-whole-string ac #t anthy-type-hiragana)))
-      (if (and (number? (anthy-context-ac-id ac))
+      (if (and ac-id
 	       (> (string-length preconv-str)
 		  0))
 	  (begin
@@ -664,7 +710,7 @@
        (direct
 	(im-commit ac direct))
 
-       ;; spcae key => commit
+       ;; space key => commit
        ((anthy-space-key? key key-state)
 	(if (anthy-context-alnum ac)
 	    (im-commit ac (list-ref
@@ -696,11 +742,13 @@
 				    (charcode->string key))
 				(if (symbol? key)
 				    (symbol->string key)
-				    (charcode->string (to-lower-char key)))))
+				    (charcode->string (ichar-downcase key)))))
 		   (res (rk-push-key! rkc key-str)))
 	      (if res
 		  (begin
-		    (ustr-insert-elem! (anthy-context-preconv-ustr ac) res)
+		    (if (list? (car res))
+		      (ustr-insert-seq! (anthy-context-preconv-ustr ac) res)
+		      (ustr-insert-elem! (anthy-context-preconv-ustr ac) res))
 		    (ustr-insert-elem! (anthy-context-raw-ustr ac)
 				       (if (and (intern-key-symbol key-str)
 						(symbol-bound?
@@ -730,6 +778,30 @@
      (else
       state))))
 
+(define anthy-learn-transposing-text
+  (lambda (ac)
+    (let ((ac-id (anthy-context-ac-id ac))
+          (transposing-type (anthy-context-transposing-type ac))
+          (preconv-str (anthy-make-whole-string ac #t anthy-type-hiragana))
+          (type #f))
+      (define (expand-segment)
+        (if (not (= (anthy-lib-get-nr-segments ac-id) 1))
+          (begin
+            (anthy-lib-resize-segment ac-id 0 1)
+            (expand-segment))))
+      (cond
+       ((= transposing-type anthy-type-hiragana)
+        (set! type anthy-candidate-type-hiragana))
+       ((= transposing-type anthy-type-katakana)
+        (set! type anthy-candidate-type-katakana)))
+      (if (and ac-id
+               (> (string-length preconv-str) 0)
+               type)
+        (begin
+          (anthy-lib-set-string ac-id preconv-str)
+          (expand-segment)
+          (anthy-lib-commit-segment ac-id 0 type))))))
+
 (define anthy-proc-transposing-state
   (lambda (ac key key-state)
     (let ((rotate-list '())
@@ -750,24 +822,27 @@
 	    (if (and lst
 		     (not (null? (cdr lst))))
 		(set! state (car (cdr lst)))
-		(set! state (anthy-rotate-transposing-alnum-type
-			     (anthy-context-transposing-type ac)
-			     (car rotate-list)))))
+		(if (not (null? rotate-list))
+		    (set! state (anthy-rotate-transposing-alnum-type
+				 (anthy-context-transposing-type ac)
+				 (car rotate-list))))))
 	  (begin
 	    (anthy-context-set-transposing! ac #t)
 	    (set! state (car rotate-list))))
 
       (cond
-       ((or
-	 (= state anthy-type-hiragana)
-	 (= state anthy-type-katakana)
-	 (= state anthy-type-halfkana))
+       ((and state
+	     (or
+	      (= state anthy-type-hiragana)
+	      (= state anthy-type-katakana)
+	      (= state anthy-type-halfkana)))
 	(anthy-context-set-transposing-type! ac state))
-       ((or
-	 (= state anthy-type-halfwidth-alnum)
-	 (= state anthy-candidate-type-upper-halfwidth-alnum)
-	 (= state anthy-type-fullwidth-alnum)
-	 (= state anthy-candidate-type-upper-fullwidth-alnum))
+       ((and state
+	     (or
+	      (= state anthy-type-halfwidth-alnum)
+	      (= state anthy-candidate-type-upper-halfwidth-alnum)
+	      (= state anthy-type-fullwidth-alnum)
+	      (= state anthy-candidate-type-upper-fullwidth-alnum)))
 	(if (not (= (anthy-context-input-rule ac) anthy-input-rule-kana))
 	    (anthy-context-set-transposing-type! ac state)))
        (else
@@ -775,6 +850,7 @@
 	 ; commit
 	 (if (anthy-commit-key? key key-state)
 	     (begin
+	       (anthy-learn-transposing-text ac)
 	       (im-commit ac (anthy-transposing-text ac))
 	       (anthy-flush ac)
 	       #f)
@@ -846,7 +922,7 @@
 	   (cur-page (if (= anthy-nr-candidate-max 0)
 			 0
 			 (quotient n anthy-nr-candidate-max)))
-	   (pageidx (- (numeral-char->number numeralc) 1))
+	   (pageidx (- (numeric-ichar->integer numeralc) 1))
 	   (compensated-pageidx (cond
 				 ((< pageidx 0) ; pressing key_0
 				  (+ pageidx 10))
@@ -866,7 +942,7 @@
 						anthy-nr-candidate-max)))))
       (if (and
 	   compensated-idx
-	   (not (= compensated-pageidx selected-pageidx)))
+	   (not (eqv? compensated-pageidx selected-pageidx)))
 	  (begin
 	    (anthy-context-set-prediction-index! ac compensated-idx)
 	    (im-select-candidate ac compensated-idx)
@@ -881,7 +957,7 @@
 	   (cur-page (if (= anthy-nr-candidate-max 0)
 			 0
 			 (quotient p-idx anthy-nr-candidate-max)))
-	   (pageidx (- (numeral-char->number numeralc) 1))
+	   (pageidx (- (numeric-ichar->integer numeralc) 1))
 	   (compensated-pageidx (cond
 				 ((< pageidx 0) ; pressing key_0
 				  (+ pageidx 10))
@@ -903,7 +979,7 @@
       #t)
      ((and
        anthy-select-prediction-by-numeral-key?
-       (numeral-char? key))
+       (ichar-numeric? key))
       (anthy-move-prediction-in-page ac key))
      ((and
        (anthy-context-prediction-index ac)
@@ -978,9 +1054,9 @@
 	       (not (shift-key-mask key-state))))
 	     ;; go back to unselected prediction
 	     (anthy-reset-prediction-window ac)
-	     (anthy-check-prediction ac))
+	     (anthy-check-prediction ac #f))
 	    ((and
-	      (numeral-char? key)
+	      (ichar-numeric? key)
 	      anthy-select-prediction-by-numeral-key?
 	      (not (anthy-prediction-select-non-existing-index? ac key)))
 	     (anthy-context-set-predicting! ac #f)
@@ -993,10 +1069,20 @@
 	  (begin
 	    (anthy-context-set-predicting! ac #f)
 	    (anthy-context-set-prediction-index! ac #f)
+	    (if (not anthy-use-prediction?)
+		(anthy-reset-prediction-window ac))
 	    (anthy-proc-input-state ac key key-state)))))))
 
 (define anthy-proc-input-state-with-preedit
   (lambda (ac key key-state)
+    (define (check-auto-conv str)
+      (and
+	str
+	anthy-auto-start-henkan?
+	(string-find japanese-auto-start-henkan-keyword-list str)
+	(begin
+	  (anthy-reset-prediction-window ac)
+	  (anthy-begin-conv ac))))
     (let ((preconv-str (anthy-context-preconv-ustr ac))
 	  (raw-str (anthy-context-raw-ustr ac))
 	  (rkc (anthy-context-rkc ac))
@@ -1008,7 +1094,11 @@
        ((anthy-begin-conv-key? key key-state)
 	(anthy-reset-prediction-window ac)
 	(anthy-begin-conv ac))
-       
+
+       ;; prediction 
+       ((anthy-next-prediction-key? key key-state)
+	(anthy-check-prediction ac #t))
+
        ;; backspace
        ((anthy-backspace-key? key key-state)
 	(if (not (rk-backspace rkc))
@@ -1019,8 +1109,8 @@
 	      (if (and
 		   (= (anthy-context-input-rule ac) anthy-input-rule-roma)
 		   (not (null? (ustr-former-seq preconv-str)))
-		   (not (char-printable?	;; check for kana
-			 (string->char
+		   (not (ichar-printable?	;; check for kana
+			 (string->ichar
 			  (car (last (ustr-former-seq preconv-str)))))))
 		  (ja-fix-deleted-raw-str-to-valid-roma! raw-str)))))
 
@@ -1175,25 +1265,6 @@
 	#f)
 
        (else	
-	;; handle "n1" sequence as "1"
-	(if (and (not (anthy-context-alnum ac))
-		 (not (alphabet-char? key))
-		 (not (string-find
-		       (rk-expect rkc)
-		       (if (= rule anthy-input-rule-kana)
-			   (if (symbol? key)
-			       (symbol->string key)
-			       (charcode->string key))
-			   (if (symbol? key)
-			       (symbol->string key)
-			       (charcode->string (to-lower-char key)))))))
-	    (let ((pend (rk-pending rkc))
-		  (residual-kana (rk-push-key-last! rkc)))
-	      (if residual-kana
-		  (begin
-		    (ustr-insert-elem! preconv-str residual-kana)
-		    (ustr-insert-elem! raw-str pend)))))
-
 	(if (anthy-context-alnum ac)
 	    (let ((key-str (if (symbol? key)
 			       (if (symbol-bound? key)
@@ -1205,22 +1276,29 @@
 	      (rk-flush rkc) ;; OK to reset rkc here.
 	      (if residual-kana
 		  (begin
-		    (ustr-insert-elem! preconv-str residual-kana)
-		    (ustr-insert-elem! raw-str pend)))
+                    (if (list? (car residual-kana))
+                      (begin
+                        (ustr-insert-seq! preconv-str residual-kana)
+                        (ustr-insert-seq! raw-str (reverse
+                                                    (string-to-list pend))))
+                      (begin
+                        (ustr-insert-elem! preconv-str residual-kana)
+                        (ustr-insert-elem! raw-str pend)))))
 	      (ustr-insert-elem! preconv-str 
 				 (if (= (anthy-context-alnum-type ac)
 					anthy-type-halfwidth-alnum)
 				     (list key-str key-str key-str)
 				     (list (ja-wide key-str) (ja-wide key-str)
 					   (ja-wide key-str))))
-	      (ustr-insert-elem! raw-str key-str))
+	      (ustr-insert-elem! raw-str key-str)
+	      (check-auto-conv key-str))
 	    (let* ((key-str (if (= rule anthy-input-rule-kana)
 	    			(if (symbol? key)
 				    (symbol->string key)
 				    (charcode->string key))
 				(if (symbol? key)
 				    (symbol->string key)
-				    (charcode->string (to-lower-char key)))))
+				    (charcode->string (ichar-downcase key)))))
 		   (pend (rk-pending rkc))
 		   (res (rk-push-key! rkc key-str)))
 	      (if (and res
@@ -1232,10 +1310,20 @@
 			(ustr-insert-elem! preconv-str res))
 		    (if (and next-pend
 			     (not (string=? next-pend "")))
-			(ustr-insert-elem! raw-str pend)
+                        (ustr-insert-seq! raw-str
+                                          (reverse (string-to-list pend)))
 			(if (list? (car res))
 			    (begin
-			      (ustr-insert-elem! raw-str pend)
+                              (if (member pend
+                                          (map car
+                                               ja-consonant-syllable-table))
+                                ;; treat consonant having more than one
+                                ;; charactear as one raw-str in this case
+                                (ustr-insert-elem! raw-str pend)
+                                (ustr-insert-seq! raw-str (reverse
+                                                            (string-to-list
+                                                              pend))))
+                              ;; assume key-str as a vowel
 			      (ustr-insert-elem!
 			       raw-str (if (and (intern-key-symbol key-str)
 						(symbol-bound?
@@ -1251,7 +1339,8 @@
 				   (intern-key-symbol key-str)
 				   (symbol-bound? (string->symbol key-str)))
 				  (symbol-value (string->symbol key-str))
-				  key-str))))))))))))))
+				  key-str)))))))
+	      (check-auto-conv (if res (car res) #f)))))))))
 
 (define anthy-context-confirm-kana!
   (lambda (ac)
@@ -1262,7 +1351,9 @@
 	       (residual-kana (rk-peek-terminal-match rkc)))
 	    (if residual-kana
 		(begin
-		  (ustr-insert-elem! preconv-str residual-kana)
+                  (if (list? (car residual-kana))
+                    (ustr-insert-seq! preconv-str residual-kana)
+                    (ustr-insert-elem! preconv-str residual-kana))
 		  (rk-flush rkc)))))))
 
 (define anthy-reset-prediction-window
@@ -1274,15 +1365,25 @@
     (anthy-context-set-prediction-index! ac #f)))
 
 (define anthy-check-prediction
-  (lambda (ac)
+  (lambda (ac force-check?)
     (if (and
 	 (not (anthy-context-converting ac))
 	 (not (anthy-context-transposing ac))
 	 (not (anthy-context-predicting ac)))
-	(let ((preconv-str
-	       (anthy-make-whole-string ac #t (anthy-context-kana-mode ac)))
-	      (ac-id (anthy-context-ac-id ac)))
-	  (if (not (string=? preconv-str ""))
+	(let* ((use-pending-rk-for-prediction? #f)
+	       (preconv-str
+		(anthy-make-whole-string ac #t (anthy-context-kana-mode ac)))
+	       (ac-id (anthy-context-ac-id ac))
+	       (preedit-len
+		(+
+		 (ustr-length (anthy-context-preconv-ustr ac))
+		 (if (not use-pending-rk-for-prediction?)
+		     0
+		     (string-length
+		      (rk-pending (anthy-context-rkc ac)))))))
+	  (if (or
+	       (>= preedit-len anthy-prediction-start-char-count)
+	       force-check?)
 	      (begin
 		(anthy-lib-set-prediction-src-string ac-id preconv-str)
 		(let ((nr (anthy-lib-get-nr-predictions ac-id)))
@@ -1303,13 +1404,14 @@
 	(anthy-proc-input-state-with-preedit ac key key-state)
 	(anthy-proc-input-state-no-preedit ac key key-state))
     (if (and
-         anthy-use-prediction?)
-	 (anthy-check-prediction ac))))
+	 anthy-use-prediction?
+	 (not (anthy-context-predicting ac)))
+	(anthy-check-prediction ac #f))))
 
 (define anthy-separator
   (lambda (ac)
-    (let ((attr (bit-or preedit-separator
-			preedit-underline)))
+    (let ((attr (bitwise-ior preedit-separator
+			     preedit-underline)))
       (if anthy-show-segment-separator?
 	  (cons attr anthy-segment-separator)
 	  #f))))
@@ -1347,8 +1449,11 @@
 	   (right-str (ustr-latter-seq raw-str))
 	   (left-str (ustr-former-seq raw-str)))
       (append left-str
-	      (if (not (null? residual-kana))
+	      (if residual-kana
+                (if (list? (car residual-kana))
+		  (reverse (string-to-list pending))
 		  (list pending))
+		  '())
 	      right-str))))
 
 (define anthy-get-raw-candidate
@@ -1365,9 +1470,11 @@
 	  (if (member (car unconv) preconv)
 	      (let ((start (list-seq-contained? preconv unconv))
 		    (len (length unconv)))
-		(if start
+		(if (and
+                      start
+                      (= (length raw-str) (length preconv))) ;; sanity check
 		    (anthy-make-raw-string
-		     (reverse (sublist raw-str start (+ start (- len 1))))
+		     (reverse (sublist-rel raw-str start len))
 		     (if (or
 			  (= cand-idx anthy-candidate-type-halfwidth-alnum)
 			  (= cand-idx
@@ -1390,7 +1497,7 @@
 	 (not (anthy-context-prediction-index ac)))
         (anthy-input-state-preedit ac)
 	(let ((cand (anthy-get-prediction-string ac)))
-	  (list (cons (bit-or preedit-reverse preedit-cursor) cand))))))
+	  (list (cons (bitwise-ior preedit-reverse preedit-cursor) cand))))))
 
 (define anthy-converting-state-preedit
   (lambda (ac)
@@ -1401,8 +1508,8 @@
       (append-map
        (lambda (seg-idx cand-idx)
 	 (let* ((attr (if (= seg-idx cur-seg)
-			  (bit-or preedit-reverse
-				  preedit-cursor)
+			  (bitwise-ior preedit-reverse
+				       preedit-cursor)
 			  preedit-underline))
 		(cand (if (> cand-idx anthy-candidate-type-halfwidth-alnum)
 			  (anthy-lib-get-nth-candidate ac-id seg-idx cand-idx)
@@ -1551,7 +1658,7 @@
 	   (cur-page (if (= anthy-nr-candidate-max 0)
 	   		 0
 			 (quotient n anthy-nr-candidate-max)))
-	   (pageidx (- (numeral-char->number numeralc) 1))
+	   (pageidx (- (numeric-ichar->integer numeralc) 1))
 	   (compensated-pageidx (cond
 				 ((< pageidx 0) ; pressing key_0
 				  (+ pageidx 10))
@@ -1700,7 +1807,7 @@
       (anthy-cancel-conv ac))
 
      ((and anthy-select-candidate-by-numeral-key?
-	   (numeral-char? key)
+	   (ichar-numeric? key)
 	   (anthy-context-candidate-window ac))
       (anthy-move-candidate-in-page ac key))
 
@@ -1721,7 +1828,7 @@
 
 (define anthy-press-key-handler
   (lambda (ac key key-state)
-    (if (control-char? key)
+    (if (ichar-control? key)
 	(im-commit-raw ac)
 	(if (anthy-context-on ac)
 	    (if (anthy-context-transposing ac)
@@ -1738,7 +1845,7 @@
 
 (define anthy-release-key-handler
   (lambda (ac key key-state)
-    (if (or (control-char? key)
+    (if (or (ichar-control? key)
 	    (not (anthy-context-on ac)))
 	;; don't discard key release event for apps
 	(anthy-commit-raw ac))))
