@@ -135,6 +135,11 @@
                      (N_ "Icon")
                      (N_ "long description will be here."))
 
+;; subgroup
+(define-custom-group 'toolbar-display
+                     (N_ "Display behavior")
+                     (N_ "long description will be here."))
+
 ;; subgroup
 (define-custom-group 'toolbar-widget
                      (N_ "Toolbar")
@@ -399,6 +404,21 @@
 		 (lambda ()
 		   (update-style uim-color-spec (symbol-value uim-color))))
 
+(define-custom 'candidate-window-style 'vertical
+  '(global visual-preference)
+  (list 'choice
+	(list 'vertical
+	      (N_ "Vertical")
+	      (N_ "long description will be here"))
+	(list 'horizontal
+	      (N_ "Horizontal")
+	      (N_ "long description will be here"))
+	(list 'table
+	      (N_ "Table style")
+	      (N_ "long description will be here")))
+  (N_ "Candidate window type")
+  (N_ "long description will be here."))
+
 ;; referred by some bridges
 (define-custom 'candidate-window-position 'caret
   '(global visual-preference)
@@ -427,6 +447,21 @@
 		   (if enable-lazy-loading?
 		       (require "lazy-load.scm"))))
 
+(define-custom 'toolbar-display-time 'always
+  '(toolbar toolbar-display)
+  (list 'choice
+        (list 'always
+              (N_ "Always")
+              (N_ "long description will be here."))
+        (list 'mode
+              (N_ "Based on mode")
+              (N_ "long description will be here."))
+        (list 'never
+              (N_ "Never")
+              (N_ "long description will be here.")))
+  (N_ "Display")
+  (N_ "long description will be here."))
+
 ;;
 ;; toolbar buttons
 ;;
@@ -577,220 +612,6 @@
                         (eq? bridge-show-with?
                              'time))))
 
-;;
-;; Annotation
-;;
-
-(define-custom 'enable-annotation? #t
-  '(annotation candwin)
-  '(boolean)
-  (N_ "Enable annotation")
-  (N_ "long description will be here."))
-
-(define-custom 'annotation-agent 'eb
-  '(annotation candwin)
-  (list 'choice
-    (list 'eb
-      (N_ "EB library")
-      (N_ "EB library"))
-    (list 'dict
-      (N_ "dict server")
-      (N_ "dict server"))
-    (list 'filter
-      (N_ "Custom filter")
-      (N_ "Custom filter"))
-    (list 'im
-      (N_ "Input method itself")
-      (N_ "long description will be here.")))
-  (N_ "Annotation agent name")
-  (N_ "long description will be here."))
-
-(custom-add-hook 'annotation-agent
-  'custom-activity-hooks
-  (lambda ()
-    enable-annotation?))
-
-(custom-add-hook 'annotation-agent
-                 'custom-set-hooks
-                 (lambda ()
-                   (annotation-unload)
-                   (annotation-load
-                     (if enable-annotation?
-                       (symbol->string annotation-agent)
-                       #f))))
-
-;; EB Library support
-(define-custom-group 'eb
-		     (N_ "EB library")
-		     (N_ "long description will be here."))
-
-;; eb-enable-for-annotation? exists only for compatibility.
-;; You shouldn't add similar variables to other annotation agents.
-(define-custom 'eb-enable-for-annotation? #f
-  '(annotation eb)
-  '(boolean)
-  (N_ "Use EB library to search annotations")
-  (N_ "long description will be here."))
-
-(custom-add-hook 'eb-enable-for-annotation?
-  'custom-activity-hooks
-  (lambda ()
-    (and enable-annotation?
-      (eq? annotation-agent 'eb))))
-
-(define-custom 'annotation-eb-dic-path
-  (string-append (sys-datadir) "/dict")
-  '(annotation eb)
-  '(pathname directory)
-  (N_ "The directory which contains EB dictionary file")
-  (N_ "long description will be here."))
-
-(custom-add-hook 'annotation-eb-dic-path
-  'custom-activity-hooks
-  (lambda ()
-    (and enable-annotation?
-         eb-enable-for-annotation?
-         (eq? annotation-agent 'eb))))
-
-;; dict server support
-(define-custom-group 'dict
-		     (N_ "dict server")
-		     (N_ "long description will be here."))
-
-(define-custom 'annotation-dict-server
-  "dict.org"
-  '(annotation dict)
-  '(string ".*")
-  (N_ "Server address of dict")
-  (N_ "long description will be here."))
-
-(define-custom 'annotation-dict-servname
-  2628
-  '(annotation dict)
-  '(integer 0 65535)
-  (N_ "Server port of dict")
-  (N_ "long description will be here."))
-
-(define-custom 'annotation-dict-database
-  "web1913"
-  '(annotation dict)
-  '(string ".*")
-  (N_ "Database name of dict")
-  (N_ "long description will be here."))
-
-(define-custom 'annotation-dict-cache-words
-  256
-  '(annotation dict)
-  '(integer 0 65535)
-  (N_ "Number of cache of annotation")
-  (N_ "long description will be here."))
-
-(custom-add-hook 'annotation-dict-server
-		 'custom-activity-hooks
-                 (lambda ()
-                   (and enable-annotation?
-                        (eq? annotation-agent 'dict))))
-
-(custom-add-hook 'annotation-dict-servname
-		 'custom-activity-hooks
-                 (lambda ()
-                   (and enable-annotation?
-                        (eq? annotation-agent 'dict))))
-
-(custom-add-hook 'annotation-dict-database
-		 'custom-activity-hooks
-                 (lambda ()
-                   (and enable-annotation?
-                        (eq? annotation-agent 'dict))))
-
-(custom-add-hook 'annotation-dict-cache-words
-		 'custom-activity-hooks
-                 (lambda ()
-                   (and enable-annotation?
-                        (eq? annotation-agent 'dict))))
-
-(define-custom-group 'filter
-		     (N_ "Custom filter")
-		     (N_ "long description will be here."))
-
-(define-custom 'annotation-filter-server-setting? 'pipe
-  '(annotation filter)
-  (list 'choice
-        (list 'unixdomain
-              (N_ "Unix domain")
-              (N_ "Use UNIX domain socket to communicate with custom filter."))
-        (list 'tcpserver
-              (N_ "TCP server")
-              (N_ "Use tcp server to communicate with custom filter"))
-        (list 'pipe
-              (N_ "Pipe")
-              (N_ "Use pipe. spawn new annotation-filter process to communicate with custom filter")))
-  (N_ "Custom filter connection setting")
-  (N_ "long description will be here."))
-
-(custom-add-hook 'annotation-filter-server-setting?
-  'custom-activity-hooks
-  (lambda ()
-    (and enable-annotation?
-         (eq? annotation-agent 'filter))))
-
-
-(define-custom 'annotation-filter-unix-domain-socket-path "/path/of/socket"
-  '(annotation filter)
-  '(pathname regular-file)
-  (N_ "Path of custom filter socket")
-  (N_ "long description will be here."))
-
-(define-custom 'annotation-filter-tcpserver-name "localhost"
-  '(annotation filter)
-  '(string ".*")
-  (N_ "Custom filter server address")
-  (N_ "long description will be here."))
-
-(define-custom 'annotation-filter-tcpserver-port 6789
-  '(annotation filter)
-  '(integer 0 65535)
-  (N_ "Custom filter server port")
-  (N_ "long description will be here."))
-
-(define-custom 'annotation-filter-command "/path/of/filter-program"
-  '(annotation filter)
-  '(pathname regular-file)
-  (N_ "Path of custom filter")
-  (N_ "long description will be here."))
-
-(custom-add-hook 'annotation-filter-unix-domain-socket-path
-		 'custom-activity-hooks
-		 (lambda ()
-                   (and enable-annotation?
-                        (eq? annotation-agent 'filter)
-                        (eq? annotation-filter-server-setting?
-                             'unixdomain))))
-
-(custom-add-hook 'annotation-filter-tcpserver-name
-		 'custom-activity-hooks
-		 (lambda ()
-                   (and enable-annotation?
-                        (eq? annotation-agent 'filter)
-                        (eq? annotation-filter-server-setting?
-                             'tcpserver))))
-
-(custom-add-hook 'annotation-filter-tcpserver-port
-		 'custom-activity-hooks
-		 (lambda ()
-                   (and enable-annotation?
-                        (eq? annotation-agent 'filter)
-                        (eq? annotation-filter-server-setting?
-                             'tcpserver))))
-
-(custom-add-hook 'annotation-filter-command
-		 'custom-activity-hooks
-                 (lambda ()
-                   (and enable-annotation?
-                        (eq? annotation-agent 'filter)
-                        (eq? annotation-filter-server-setting?
-                             'pipe))))
-
 ;;
 ;; uim-xim specific custom
 ;;
