@@ -38,36 +38,38 @@
   '())
 
 (define imsw-iconic-label-alist
-  '((direct      . "-")
-    (ajax-ime    . "Aj")
-    (anthy       . "An")
-    (anthy-utf8  . "An")
-    (byeoru      . "B")
-    (canna       . "Ca")
-    (chewing     . "Ch")
-    (elatin      . "E")
-    (hangul2     . "H2")
-    (hangul3     . "H3")
-    (ipa-x-sampa . "I")
-    (latin       . "Lt")
-    (look        . "Lo")
-    (mana        . "Ma")
-    (mozc        . "Mz")
-    (pinyin-big5 . "Pi")
-    (pyunihan    . "Py")
-    (prime       . "Pm")
-    (romaja      . "R")
-    (sj3         . "Sj")
-    (skk         . "Sk")
-    (social-ime  . "Si")
-    (tcode       . "Tc")
-    (tutcode     . "Tu")
-    (trycode     . "Tr")
-    (viqr        . "V")
-    (wb86        . "Wb")
-    (wnn         . "Wn")
-    (yahoo-jp    . "Yj")
-    (zm          . "Zm")))
+  '((direct           . "-")
+    (ajax-ime         . "Aj")
+    (anthy            . "An")
+    (anthy-utf8       . "An")
+    (byeoru           . "B")
+    (baidu-olime-jp   . "Bj")
+    (canna            . "Ca")
+    (chewing          . "Ch")
+    (elatin           . "E")
+    (hangul2          . "H2")
+    (hangul3          . "H3")
+    (ipa-x-sampa      . "I")
+    (latin            . "Lt")
+    (look             . "Lo")
+    (mana             . "Ma")
+    (mozc             . "Mz")
+    (pinyin-big5      . "Pi")
+    (pyunihan         . "Py")
+    (prime            . "Pm")
+    (romaja           . "R")
+    (sj3              . "Sj")
+    (skk              . "Sk")
+    (social-ime       . "Si")
+    (tcode            . "Tc")
+    (tutcode          . "Tu")
+    (trycode          . "Tr")
+    (viqr             . "V")
+    (wb86             . "Wb")
+    (wnn              . "Wn")
+    (yahoo-jp         . "Yj")
+    (google-cgiapi-jp . "Gj")
+    (zm               . "Zm")))
 
 (define imsw-default-iconic-label "IM")
 
