@@ -30,365 +30,9 @@
 
 ;; Japanese EUC
 
+(require-extension (srfi 1 2))
 (require-custom "japanese-custom.scm")
 
-(define ja-rk-rule-basic
-  '(
-    ((("-"). ())("" "" ""))
-    (((","). ())("" "" ""))
-    ((("."). ())("" "" ""))
-    ((("!"). ())("" "" "!"))
-    ((("\""). ())("" "" "\""))
-    ((("#"). ())("" "" "#"))
-    ((("$"). ())("" "" "$"))
-    ((("%"). ())("" "" "%"))
-    ((("&"). ())("" "" "&"))
-    ((("'"). ())("" "" "'"))
-    ((("("). ())("" "" "("))
-    (((")"). ())("" "" ")"))
-    ((("~"). ())("" "" "~"))
-    ((("="). ())("" "" "="))
-    ((("^"). ())("" "" "^"))
-    ((("\\"). ())("" "" "\\"))
-    ((("|"). ())("" "" "|"))
-    ((("`"). ())("" "" "`"))
-    ((("@"). ())("" "" "@"))
-    ((("{"). ())("" "" "{"))
-    ((("["). ())("" "" ""))
-    ((("+"). ())("" "" "+"))
-    (((";"). ())("" "" ";"))
-    ((("*"). ())("" "" "*"))
-    (((":"). ())("" "" ":"))
-    ((("}"). ())("" "" "}"))
-    ((("]"). ())("" "" ""))
-    ((("<"). ())("" "" "<"))
-    (((">"). ())("" "" ">"))
-    ((("?"). ())("" "" "?"))
-    ((("/"). ())("" "" "/"))
-    ((("_"). ())("" "" "_"))
-    ;; Since ordinary Japanese users press the "yen sign" key on
-    ;; Japanese keyboard in romaji-halfwidth-kana-mode "to input
-    ;; character code 134" rather than "to input yen sign symbol", I
-    ;; changed the fullwidth yen sign with backslash.
-    ;;   -- YamaKen 2007-09-17
-    ;; ((("yen"). ())("" "" "")) ;; XXX
-    ((("yen"). ())("" "" "\\"))
-
-    ((("1"). ())("1" "1" "1"))
-    ((("2"). ())("2" "2" "2"))
-    ((("3"). ())("3" "3" "3"))
-    ((("4"). ())("4" "4" "4"))
-    ((("5"). ())("5" "5" "5"))
-    ((("6"). ())("6" "6" "6"))
-    ((("7"). ())("7" "7" "7"))
-    ((("8"). ())("8" "8" "8"))
-    ((("9"). ())("9" "9" "9"))
-    ((("0"). ())("0" "0" "0"))
-
-    ((("a"). ())("" "" ""))
-    ((("i"). ())("" "" ""))
-    ((("u"). ())("" "" ""))
-    ((("e"). ())("" "" ""))
-    ((("o"). ())("" "" ""))
-
-    ((("x" "a"). ())("" "" ""))
-    ((("x" "i"). ())("" "" ""))
-    ((("x" "y" "i"). ())("" "" ""))
-    ((("x" "u"). ())("" "" ""))
-    ((("x" "e"). ())("" "" ""))
-    ((("x" "y" "e"). ())("" "" ""))
-    ((("x" "o"). ())("" "" ""))
-
-    ((("l" "a"). ())("" "" ""))
-    ((("l" "i"). ())("" "" ""))
-    ((("l" "u"). ())("" "" ""))
-    ((("l" "e"). ())("" "" ""))
-    ((("l" "o"). ())("" "" ""))
-
-    ((("k" "k"). ("k"))("" "" ""))
-
-    ((("k" "a"). ())("" "" ""))
-    ((("k" "i"). ())("" "" ""))
-    ((("k" "u"). ())("" "" ""))
-    ((("k" "e"). ())("" "" ""))
-    ((("k" "o"). ())("" "" ""))
-    ((("k" "y" "a"). ())(("" "" "") ("" "" "")))
-    ((("k" "y" "i"). ())(("" "" "") ("" "" "")))
-    ((("k" "y" "u"). ())(("" "" "") ("" "" "")))
-    ((("k" "y" "e"). ())(("" "" "") ("" "" "")))
-    ((("k" "y" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("g" "g"). ("g"))("" "" ""))
-
-
-    ((("g" "a"). ())("" "" ""))
-    ((("g" "i"). ())("" "" ""))
-    ((("g" "u"). ())("" "" ""))
-    ((("g" "e"). ())("" "" ""))
-    ((("g" "o"). ())("" "" ""))
-
-    ((("g" "y" "a"). ())(("" "" "") ("" "" "")))
-    ((("g" "y" "i"). ())(("" "" "") ("" "" "")))
-    ((("g" "y" "u"). ())(("" "" "") ("" "" "")))
-    ((("g" "y" "e"). ())(("" "" "") ("" "" "")))
-    ((("g" "y" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("q" "a"). ())(("" "" "") ("" "" "")))
-    ((("q" "i"). ())(("" "" "") ("" "" "")))
-    ((("q" "u"). ())("" "" ""))
-    ((("q" "e"). ())(("" "" "") ("" "" "")))
-    ((("q" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("s" "s"). ("s"))("" "" ""))
-
-    ((("s" "a"). ())("" "" ""))
-    ((("s" "i"). ())("" "" ""))
-    ((("s" "u"). ())("" "" ""))
-    ((("s" "e"). ())("" "" ""))
-    ((("s" "o"). ())("" "" ""))
-
-    ((("s" "y" "a"). ())(("" "" "") ("" "" "")))
-    ((("s" "y" "i"). ())(("" "" "") ("" "" "")))
-    ((("s" "y" "u"). ())(("" "" "") ("" "" "")))
-    ((("s" "y" "e"). ())(("" "" "") ("" "" "")))
-    ((("s" "y" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("z" "z"). ("z"))("" "" ""))
-
-    ((("z" "a"). ())("" "" ""))
-    ((("z" "i"). ())("" "" ""))
-    ((("z" "u"). ())("" "" ""))
-    ((("z" "e"). ())("" "" ""))
-    ((("z" "o"). ())("" "" ""))
-    ((("z" "y" "a"). ())(("" "" "") ("" "" "")))
-    ((("z" "y" "i"). ())(("" "" "") ("" "" "")))
-    ((("z" "y" "u"). ())(("" "" "") ("" "" "")))
-    ((("z" "y" "e"). ())(("" "" "") ("" "" "")))
-    ((("z" "y" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("j" "j"). ("j"))("" "" ""))
-
-    ((("j" "a"). ())(("" "" "") ("" "" "")))
-    ((("j" "i"). ())("" "" ""))
-    ((("j" "u"). ())(("" "" "") ("" "" "")))
-    ((("j" "e"). ())(("" "" "") ("" "" "")))
-    ((("j" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("j" "y" "a"). ())(("" "" "") ("" "" "")))
-    ((("j" "y" "i"). ())(("" "" "") ("" "" "")))
-    ((("j" "y" "u"). ())(("" "" "") ("" "" "")))
-    ((("j" "y" "e"). ())(("" "" "") ("" "" "")))
-    ((("j" "y" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("t" "t"). ("t"))("" "" ""))
-    ((("t" "c"). ("c"))("" "" ""))
-
-    ((("t" "a"). ())("" "" ""))
-    ((("t" "i"). ())("" "" ""))
-    ((("t" "u"). ())("" "" ""))
-    ((("t" "e"). ())("" "" ""))
-    ((("t" "o"). ())("" "" ""))
-
-    ((("t" "y" "a"). ())(("" "" "") ("" "" "")))
-    ((("t" "y" "i"). ())(("" "" "") ("" "" "")))
-    ((("t" "y" "u"). ())(("" "" "") ("" "" "")))
-    ((("t" "y" "e"). ())(("" "" "") ("" "" "")))
-    ((("t" "y" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("t" "s" "a"). ())(("" "" "") ("" "" "")))
-    ((("t" "s" "i"). ())(("" "" "") ("" "" "")))
-    ((("t" "s" "u"). ())("" "" ""))
-    ((("t" "s" "e"). ())(("" "" "") ("" "" "")))
-    ((("t" "s" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("c" "y" "a"). ())(("" "" "") ("" "" "")))
-    ((("c" "y" "i"). ())(("" "" "") ("" "" "")))
-    ((("c" "y" "u"). ())(("" "" "") ("" "" "")))
-    ((("c" "y" "e"). ())(("" "" "") ("" "" "")))
-    ((("c" "y" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("x" "t" "u"). ())("" "" ""))
-    ((("x" "t" "s" "u"). ())("" "" ""))
-    ((("c" "c"). ("c"))("" "" ""))
-
-    ((("d" "d"). ("d"))("" "" ""))
-
-    ((("d" "a"). ())("" "" ""))
-    ((("d" "i"). ())("" "" ""))
-    ((("d" "u"). ())("" "" ""))
-    ((("d" "e"). ())("" "" "Î"))
-    ((("d" "o"). ())("" "" "Ď"))
-
-    ((("d" "y" "a"). ())(("" "" "") ("" "" "")))
-    ((("d" "y" "i"). ())(("" "" "") ("" "" "")))
-    ((("d" "y" "u"). ())(("" "" "") ("" "" "")))
-    ((("d" "y" "e"). ())(("" "" "") ("" "" "")))
-    ((("d" "y" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("n" "n"). ())("" "" ""))
-    ((("n" "'"). ())("" "" ""))
-    ((("n"). ())("" "" ""))
-
-    ((("n" "a"). ())("" "" ""))
-    ((("n" "i"). ())("" "" ""))
-    ((("n" "u"). ())("" "" ""))
-    ((("n" "e"). ())("" "" ""))
-    ((("n" "o"). ())("" "" ""))
-
-    ((("n" "y" "a"). ())(("" "" "") ("" "" "")))
-    ((("n" "y" "i"). ())(("" "" "") ("" "" "")))
-    ((("n" "y" "u"). ())(("" "" "") ("" "" "")))
-    ((("n" "y" "e"). ())(("" "" "") ("" "" "")))
-    ((("n" "y" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("h" "h"). ("h"))("" "" ""))
-
-    ((("h" "a"). ())("" "" ""))
-    ((("h" "i"). ())("" "" ""))
-    ((("h" "u"). ())("" "" ""))
-    ((("h" "e"). ())("" "" ""))
-    ((("h" "o"). ())("" "" ""))
-
-    ((("h" "y" "a"). ())(("" "" "") ("" "" "")))
-    ((("h" "y" "i"). ())(("" "" "") ("" "" "")))
-    ((("h" "y" "u"). ())(("" "" "") ("" "" "")))
-    ((("h" "y" "e"). ())(("" "" "") ("" "" "")))
-    ((("h" "y" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("f" "f"). ("f"))("" "" ""))
-
-    ((("f" "a"). ())(("" "" "") ("" "" "")))
-    ((("f" "i"). ())(("" "" "") ("" "" "")))
-    ((("f" "u"). ())("" "" ""))
-    ((("f" "e"). ())(("" "" "") ("" "" "")))
-    ((("f" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("f" "y" "a"). ())(("" "" "") ("" "" "")))
-    ((("f" "y" "i"). ())(("" "" "") ("" "" "")))
-    ((("f" "y" "u"). ())(("" "" "") ("" "" "")))
-    ((("f" "y" "e"). ())(("" "" "") ("" "" "")))
-    ((("f" "y" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("b" "b"). ("b"))("" "" ""))
-
-    ((("b" "a"). ())("" "" "ʎ"))
-    ((("b" "i"). ())("" "" "ˎ"))
-    ((("b" "u"). ())("" "" "̎"))
-    ((("b" "e"). ())("" "" "͎"))
-    ((("b" "o"). ())("" "" "Ύ"))
-
-    ((("b" "y" "a"). ())(("" "" "ˎ") ("" "" "")))
-    ((("b" "y" "i"). ())(("" "" "ˎ") ("" "" "")))
-    ((("b" "y" "u"). ())(("" "" "ˎ") ("" "" "")))
-    ((("b" "y" "e"). ())(("" "" "ˎ") ("" "" "")))
-    ((("b" "y" "o"). ())(("" "" "ˎ") ("" "" "")))
-
-    ((("p" "p"). ("p"))("" "" ""))
-
-    ((("p" "a"). ())("" "" "ʎ"))
-    ((("p" "i"). ())("" "" "ˎ"))
-    ((("p" "u"). ())("" "" "̎"))
-    ((("p" "e"). ())("" "" "͎"))
-    ((("p" "o"). ())("" "" "Ύ"))
-
-    ((("p" "y" "a"). ())(("" "" "ˎ") ("" "" "")))
-    ((("p" "y" "i"). ())(("" "" "ˎ") ("" "" "")))
-    ((("p" "y" "u"). ())(("" "" "ˎ") ("" "" "")))
-    ((("p" "y" "e"). ())(("" "" "ˎ") ("" "" "")))
-    ((("p" "y" "o"). ())(("" "" "ˎ") ("" "" "")))
-
-    ((("m" "m"). ("m"))("" "" ""))
-
-    ((("m" "b"). ("b"))("" "" ""))
-    ((("m" "p"). ("p"))("" "" ""))
-
-    ((("m" "a"). ())("" "" ""))
-    ((("m" "i"). ())("" "" ""))
-    ((("m" "u"). ())("" "" ""))
-    ((("m" "e"). ())("" "" ""))
-    ((("m" "o"). ())("" "" ""))
-
-    ((("m" "y" "a"). ())(("" "" "") ("" "" "")))
-    ((("m" "y" "i"). ())(("" "" "") ("" "" "")))
-    ((("m" "y" "u"). ())(("" "" "") ("" "" "")))
-    ((("m" "y" "e"). ())(("" "" "") ("" "" "")))
-    ((("m" "y" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("y" "y"). ("y"))("" "" ""))
-
-    ((("y" "a"). ())("" "" ""))
-    ((("y" "u"). ())("" "" ""))
-    ((("y" "e"). ())(("" "" "") ("" "" "")))
-    ((("y" "o"). ())("" "" ""))
-
-    ((("x" "c" "a"). ())("" "" ""))
-    ((("x" "k" "a"). ())("" "" ""))
-    ((("x" "k" "e"). ())("" "" ""))
-
-    ((("x" "y" "a"). ())("" "" ""))
-    ((("x" "y" "u"). ())("" "" ""))
-    ((("x" "y" "o"). ())("" "" ""))
-
-    ((("r" "r"). ("r"))("" "" ""))
-
-    ((("r" "a"). ())("" "" ""))
-    ((("r" "i"). ())("" "" ""))
-    ((("r" "u"). ())("" "" ""))
-    ((("r" "e"). ())("" "" ""))
-    ((("r" "o"). ())("" "" ""))
-
-    ((("l" "t" "u"). ())("" "" ""))
-    ((("l" "t" "s" "u"). ())("" "" ""))
-
-    ((("l" "y" "a"). ())("" "" ""))
-    ((("l" "y" "i"). ())("" "" ""))
-    ((("l" "y" "u"). ())("" "" ""))
-    ((("l" "y" "e"). ())("" "" ""))
-    ((("l" "y" "o"). ())("" "" ""))
-
-    ((("r" "y" "a"). ())(("" "" "") ("" "" "")))
-    ((("r" "y" "i"). ())(("" "" "") ("" "" "")))
-    ((("r" "y" "u"). ())(("" "" "") ("" "" "")))
-    ((("r" "y" "e"). ())(("" "" "") ("" "" "")))
-    ((("r" "y" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("w" "w"). ("w"))("" "" ""))
-
-    ((("w" "a"). ())("" "" ""))
-    ((("w" "i"). ())(("" "" "") ("" "" "")))
-    ((("w" "u"). ())("" "" ""))
-    ((("w" "e"). ())(("" "" "") ("" "" "")))
-    ((("w" "o"). ())("" "" ""))
-    ((("w" "h" "a"). ())(("" "" "") ("" "" "")))
-    ((("w" "h" "i"). ())(("" "" "") ("" "" "")))
-    ((("w" "h" "u"). ())("" "" ""))
-    ((("w" "h" "e"). ())(("" "" "") ("" "" "")))
-    ((("w" "h" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("v" "v"). ("v"))("" "" ""))
-
-    ((("v" "a"). ())(("" "" "") ("" "" "")))
-    ((("v" "i"). ())(("" "" "") ("" "" "")))
-    ((("v" "u"). ())("" "" ""))
-    ((("v" "e"). ())(("" "" "") ("" "" "")))
-    ((("v" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("v" "y" "a"). ())(("" "" "") ("" "" "")))
-    ((("v" "y" "u"). ())(("" "" "") ("" "" "")))
-    ((("v" "y" "o"). ())(("" "" "") ("" "" "")))
-
-    ((("z" "k"). ())("" "" ""))
-    ((("z" "j"). ())("" "" ""))
-    ((("z" "h"). ())("" "" ""))
-    ((("z" "l"). ())("" "" ""))
-    ((("z" "-"). ())("" "" ""))
-    ((("z" "["). ())("" "" ""))
-    ((("z" "]"). ())("" "" ""))
-    ((("z" ","). ())("" "" ""))
-    ((("z" "."). ())("" "" ""))
-    ((("z" "/"). ())("" "" ""))
-    ))
-
 (define ja-rk-rule-additional
   '(
     ((("d" "s" "u"). ())("" "" ""))
@@ -528,7 +172,7 @@
     ((("t" "'" "y" "u"). ())(("" "" "") ("" "" "")))
 
     ((("t" "'" "u"). ())(("" "" "") ("" "" "")))
- 
+
     ))
 
 (define ja-rk-rule (append ja-rk-rule-basic ja-rk-rule-additional))
@@ -897,7 +541,7 @@
 	  (string-append (ja-make-kana-str (cdr sl) type)
 			 (get-str-by-type sl))
 	  ""))))
-    
+
 (define japanese-roma-set-yen-representation
   (lambda ()
     ;; Since ordinary Japanese users press the "yen sign" key on
@@ -921,7 +565,7 @@
        (filter (lambda (x) (not (string=? "n" x)))
                (map car ja-consonant-syllable-table))))
 
-(define ja-rk-rule-update
+(define ja-rk-rule-keep-consonant-update
   (lambda ()
     (if ja-rk-rule-keep-consonant?
       (set! ja-rk-rule (append ja-rk-rule-consonant-to-keep
@@ -930,6 +574,103 @@
       (set! ja-rk-rule (append ja-rk-rule-basic
                                ja-rk-rule-additional)))))
 
+;; In ja-rk-rule-update,
+;; don't set ja-rk-rule-basic to ja-rk-rule-basic-uim
+;;
+;; If you do so, when the following conditions are met
+;;   1) users call ja-rk-rule-update in ~/.uim
+;;   2) ja-rk-rule-type is 'uim (default value)
+;; ja-rk-rule-basic is always overridden
+;; with ja-rk-rule-basic-uim.
+;; This is an unwanted behavior for users.
+(define ja-rk-rule-update
+  (lambda ()
+    (and
+      (eq? ja-rk-rule-type 'custom)
+      (set! ja-rk-rule-basic
+        (ja-rk-rule-table->rule ja-rk-rule-table-basic)))
+    (ja-rk-rule-keep-consonant-update)))
+
+;;; Convert EUC-JP code to EUC-JP string (cf. ucs->utf8-string in ichar.scm)
+(define (ja-euc-jp-code->euc-jp-string code)
+  (with-char-codec "EUC-JP"
+    (lambda ()
+      (let ((str (list->string (list (integer->char code)))))
+        (with-char-codec "ISO-8859-1"
+          (lambda ()
+            (%%string-reconstruct! str)))))))
+
+;;; Convert JIS code(ISO-2022-JP) to EUC-JP string
+(define (ja-jis-code->euc-jp-string state jis1 jis2)
+  (let
+    ((ej0 (if (eq? state 'jisx0213-plane2) #x8f 0))
+     (ej1 (+ jis1 #x80))
+     (ej2 (+ jis2 #x80)))
+    (and
+      ;; sigscheme/src/encoding.c:eucjp_int2str()
+      (<= #xa1 ej1 #xfe) ; IN_GR94()
+      (if (= ej0 #x8f) ; SS3?
+        (<= #xa1 ej2 #xfe) ; IN_GR94()
+        (<= #xa0 ej2 #xff)) ; IN_GR96()
+      (ja-euc-jp-code->euc-jp-string
+        (+ (* ej0 #x10000) (* ej1 #x100) ej2)))))
+
+;;; Convert reverse string list of JIS code to one EUC-JP kanji string
+;;; ("d" "2" "0" "5") -> "Э"
+(define (ja-kanji-code-input-jis str-list)
+  (and-let*
+    ((length=4? (= (length str-list) 4))
+     (str1 (string-list-concat (take-right str-list 2)))
+     (str2 (string-list-concat (take str-list 2)))
+     (jis1 (string->number str1 16))
+     (jis2 (string->number str2 16)))
+    (ja-jis-code->euc-jp-string 'jisx0213-plane1 jis1 jis2)))
+
+;;; Convert reverse string list of Kuten code to one EUC-JP kanji string
+;;; ("3" "1" "-" "8" "4" "-" "1") -> "Э"
+(define (ja-kanji-code-input-kuten str-list)
+  (let*
+    ((numlist (string-split (string-list-concat str-list) "-"))
+     (men-exists? (>= (length numlist) 3))
+     (men (if men-exists? (string->number (list-ref numlist 0)) 1))
+     (ku (string->number (list-ref numlist (if men-exists? 1 0))))
+     (ten (string->number (list-ref numlist (if men-exists? 2 1)))))
+    (and men ku ten (<= 1 men 2)
+      (ja-jis-code->euc-jp-string
+        (if (= men 2) 'jisx0213-plane2 'jisx0213-plane1)
+        (+ ku #x20) (+ ten #x20)))))
+
+;;; Convert reverse string list of UCS to one EUC-JP kanji string
+;;; ("5" "8" "E" "4" "+" "U") -> "Э"
+(define (ja-kanji-code-input-ucs str-list)
+  (and-let*
+    ((str-list-1 (drop-right str-list 1)) ; drop last "U"
+     (not-only-u? (not (null? str-list-1)))
+     (ucs-str (if (string=? (last str-list-1) "+")
+                (drop-right str-list-1 1)
+                str-list-1))
+     (ucs (string->number (string-list-concat ucs-str) 16))
+     ;; range check to avoid error
+     (valid? ; sigscheme/src/sigschemeinternal.h:ICHAR_VALID_UNICODEP()
+      (or
+        (<= 0 ucs #xd7ff)
+        (<= #xe000 ucs #x10ffff)))
+     (utf8-str (ucs->utf8-string ucs))
+     (ic (iconv-open "EUC-JP" "UTF-8")))
+    (let ((eucj-str (iconv-code-conv ic utf8-str)))
+      (iconv-release ic)
+      eucj-str)))
+
+;;; Convert reverse string list to one EUC-JP kanji string
+(define (ja-kanji-code-input str-list)
+  (cond
+    ((string-ci=? (last str-list) "u")
+      (ja-kanji-code-input-ucs str-list))
+    ((member "-" str-list)
+      (ja-kanji-code-input-kuten str-list))
+    (else
+      (ja-kanji-code-input-jis str-list))))
+
 ;;
 (require "rk.scm")
 
