@@ -29,6 +29,7 @@
 ;;;;
 
 (require-custom "generic-key-custom.scm")
+(require-custom "m17nlib-custom.scm")
 
 ;;; user configs
 
@@ -38,20 +39,6 @@
 (define-key m17nlib-on-key? 'generic-on-key?)
 (define-key m17nlib-off-key? 'generic-off-key?)
 
-;; widgets and actions
-
-;; widgets
-(define m17nlib-widgets '(widget_m17nlib_input_mode))
-
-;; default activity for each widgets
-(define default-widget_m17nlib_input_mode 'action_m17nlib_off)
-
-;; actions of widget_m17nlib_input_mode
-(define m17nlib-input-mode-actions
-  '(action_m17nlib_off
-    action_m17nlib_on))
-
-
 ;;; implementations
 
 (register-action 'action_m17nlib_off
@@ -305,7 +292,10 @@
 
 (define m17nlib-release-key-handler
   (lambda (mc key key-state)
-    #f))
+    (if (or (ichar-control? key)
+            (not (m17nlib-context-on mc)))
+      ;; don't discard key release event for apps
+      (m17nlib-commit-raw mc))))
 
 (define m17nlib-reset-handler
   (lambda (mc)
