@@ -86,6 +86,12 @@
   (N_ "Enable learning in mazegaki conversion")
   (N_ "long description will be here."))
 
+(define-custom 'tutcode-mazegaki-fixed-priority-count 0
+  '(tutcode tutcode-mazegaki)
+  '(integer 0 65535)
+  (N_ "Number of candidates to be excluded from mazegaki learning")
+  (N_ "long description will be here."))
+
 (define-custom 'tutcode-use-recursive-learning? #t
   '(tutcode tutcode-mazegaki)
   '(boolean)
@@ -98,6 +104,12 @@
   (N_ "Enable vi-cooperative mode")
   (N_ "long description will be here."))
 
+(define-custom 'tutcode-show-pending-rk? #f
+  '(tutcode)
+  '(boolean)
+  (N_ "Show pending key sequences")
+  (N_ "long description will be here."))
+
 (define-custom 'tutcode-use-dvorak? #f
   '(tutcode)
   '(boolean)
@@ -116,6 +128,18 @@
   (N_ "Enable fallback of surrounding text API")
   (N_ "long description will be here."))
 
+(define-custom 'tutcode-keep-illegal-sequence? #f
+  '(tutcode)
+  '(boolean)
+  (N_ "Keep key sequence not convertible to Kanji")
+  (N_ "long description will be here."))
+
+(define-custom 'tutcode-delete-leading-delimiter-on-postfix-kanji2seq? #f
+  '(tutcode)
+  '(boolean)
+  (N_ "Delete leading delimiter on postfix kanji to sequence conversion")
+  (N_ "long description will be here."))
+
 (define-custom 'tutcode-history-size 0
   '(tutcode)
   '(integer 0 65535)
@@ -140,6 +164,17 @@
   (N_ "Maximum length of yomi suffix for mazegaki conversion")
   (N_ "long description will be here."))
 
+(define-custom 'tutcode-bushu-conversion-algorithm 'tc-2.1+ml1925
+  '(tutcode tutcode-bushu)
+  (list 'choice
+    (list 'tc-2.1+ml1925
+          (N_ "tc-2.1+[tcode-ml:1925]") (N_ "tc-2.1+[tcode-ml:1925]"))
+    (list 'kw-yamanobe
+          (N_ "Kanchoku Win YAMANOBE") (N_ "Kanchoku Win YAMANOBE"))
+    (list 'tc-2.3.1-22.6 (N_ "tc-2.3.1-22.6") (N_ "tc-2.3.1-22.6")))
+  (N_ "Bushu conversion algorithm")
+  (N_ "long description will be here."))
+
 (define-custom 'tutcode-use-interactive-bushu-conversion? #f
   '(tutcode tutcode-bushu)
   '(boolean)
@@ -176,10 +211,10 @@
   (N_ "Use candidate window")
   (N_ "long description will be here."))
 
-(define-custom 'tutcode-use-table-style-candidate-window? #f
+(define-custom 'tutcode-use-pseudo-table-style? #f
   '(tutcode candwin)
   '(boolean)
-  (N_ "Use table style candidate window")
+  (N_ "Use pseudo table style layout")
   (N_ "long description will be here."))
 
 (define-custom 'tutcode-candidate-window-table-layout 'qwerty-jis
@@ -191,9 +226,15 @@
   (N_ "Key layout of table style candidate window")
   (N_ "long description will be here."))
 
-(define-custom 'tutcode-commit-candidate-by-label-key? #t
+(define-custom 'tutcode-commit-candidate-by-label-key 'always
   '(tutcode candwin)
-  '(boolean)
+  (list 'choice
+	(list 'always (N_ "always") (N_ "All keys as label key"))
+	(list 'havecand (N_ "which have candidate")
+          (N_ "Enable keys which have candidate"))
+	(list 'candwin (N_ "while candidate window is shown")
+          (N_ "Enable while candidate window is shown"))
+	(list 'never (N_ "never") (N_ "Never")))
   (N_ "Commit candidate by heading label keys")
   (N_ "long description will be here."))
 
@@ -239,6 +280,12 @@
   (N_ "Use stroke help window")
   (N_ "long description will be here."))
 
+(define-custom 'tutcode-show-stroke-help-window-on-no-input? #t
+  '(tutcode candwin)
+  '(boolean)
+  (N_ "Show stroke help window on no input")
+  (N_ "long description will be here."))
+
 (define-custom 'tutcode-use-auto-help-window? #f
   '(tutcode candwin)
   '(boolean)
@@ -251,6 +298,54 @@
   (N_ "Show real keys on auto help window")
   (N_ "long description will be here."))
 
+(define-custom 'tutcode-candidate-window-use-delay? #f
+  '(tutcode candwin)
+  '(boolean)
+  (N_ "Use delay showing candidate window")
+  (N_ "long description will be here."))
+
+(define-custom 'tutcode-candidate-window-activate-delay-for-mazegaki 0
+  '(tutcode candwin)
+  '(integer 0 65535)
+  (N_ "Delay before showing candidate window for mazegaki [s]")
+  (N_ "long description will be here."))
+
+(define-custom 'tutcode-candidate-window-activate-delay-for-stroke-help 2
+  '(tutcode candwin)
+  '(integer 0 65535)
+  (N_ "Delay before showing candidate window for stroke help [s]")
+  (N_ "long description will be here."))
+
+(define-custom 'tutcode-candidate-window-activate-delay-for-auto-help 1
+  '(tutcode candwin)
+  '(integer 0 65535)
+  (N_ "Delay before showing candidate window for auto help [s]")
+  (N_ "long description will be here."))
+
+(define-custom 'tutcode-candidate-window-activate-delay-for-completion 2
+  '(tutcode candwin)
+  '(integer 0 65535)
+  (N_ "Delay before showing candidate window for completion [s]")
+  (N_ "long description will be here."))
+
+(define-custom 'tutcode-candidate-window-activate-delay-for-prediction 2
+  '(tutcode candwin)
+  '(integer 0 65535)
+  (N_ "Delay before showing candidate window for prediction [s]")
+  (N_ "long description will be here."))
+
+(define-custom 'tutcode-candidate-window-activate-delay-for-bushu-prediction 2
+  '(tutcode candwin)
+  '(integer 0 65535)
+  (N_ "Delay before showing candidate window for bushu prediction [s]")
+  (N_ "long description will be here."))
+
+(define-custom 'tutcode-candidate-window-activate-delay-for-interactive-bushu 1
+  '(tutcode candwin)
+  '(integer 0 65535)
+  (N_ "Delay before showing candidate window for interactive bushu conversion [s]")
+  (N_ "long description will be here."))
+
 ;; prediction/completion
 (define-custom 'tutcode-use-completion? #f
   '(tutcode tutcode-prediction)
@@ -304,6 +399,11 @@
   (N_ "long description will be here."))
 
 ;; activity dependency
+(custom-add-hook 'tutcode-mazegaki-fixed-priority-count
+                 'custom-activity-hooks
+                 (lambda ()
+                   tutcode-enable-mazegaki-learning?))
+
 (custom-add-hook 'tutcode-candidate-op-count
 		 'custom-activity-hooks
 		 (lambda ()
@@ -319,12 +419,7 @@
 		 (lambda ()
 		   tutcode-use-candidate-window?))
 
-(custom-add-hook 'tutcode-nr-candidate-max-for-prediction
-		 'custom-activity-hooks
-		 (lambda ()
-		   tutcode-use-candidate-window?))
-
-(custom-add-hook 'tutcode-nr-candidate-max-for-guide
+(custom-add-hook 'tutcode-nr-candidate-max-for-history
 		 'custom-activity-hooks
 		 (lambda ()
 		   tutcode-use-candidate-window?))
@@ -334,40 +429,54 @@
 		 (lambda ()
 		   tutcode-use-auto-help-window?))
 
-(custom-add-hook 'tutcode-use-table-style-candidate-window?
+(define (tutcode-custom-adjust-nr-candidate-max)
+  (if (or (eq? candidate-window-style 'table) tutcode-use-pseudo-table-style?)
+    (begin
+      (custom-set-value! 'tutcode-nr-candidate-max
+        (length tutcode-table-heading-label-char-list))
+      (custom-set-value!
+        'tutcode-nr-candidate-max-for-kigou-mode
+        (length tutcode-table-heading-label-char-list-for-kigou-mode))
+      (custom-set-value!
+        'tutcode-nr-candidate-max-for-prediction
+        (length tutcode-heading-label-char-list-for-prediction))
+      (custom-set-value!
+        'tutcode-nr-candidate-max-for-guide
+        (- (length tutcode-table-heading-label-char-list-for-kigou-mode)
+           (length tutcode-heading-label-char-list-for-prediction)))
+      (custom-set-value!
+        'tutcode-nr-candidate-max-for-history
+        (length tutcode-table-heading-label-char-list)))
+    (begin
+      (custom-set-value! 'tutcode-nr-candidate-max 10)
+      (custom-set-value! 'tutcode-nr-candidate-max-for-kigou-mode 10)
+      (custom-set-value! 'tutcode-nr-candidate-max-for-prediction 10)
+      (custom-set-value! 'tutcode-nr-candidate-max-for-guide 10)
+      (custom-set-value! 'tutcode-nr-candidate-max-for-history 10))))
+
+(custom-add-hook 'candidate-window-style
   'custom-set-hooks
-  (lambda ()
-    (if tutcode-use-table-style-candidate-window?
-      (begin
-        (custom-set-value! 'tutcode-nr-candidate-max
-          (length tutcode-table-heading-label-char-list))
-        (custom-set-value!
-          'tutcode-nr-candidate-max-for-kigou-mode
-          (length tutcode-table-heading-label-char-list-for-kigou-mode))
-        (custom-set-value!
-          'tutcode-nr-candidate-max-for-prediction
-          (length tutcode-heading-label-char-list-for-prediction))
-        (custom-set-value!
-          'tutcode-nr-candidate-max-for-guide
-          (- (length tutcode-table-heading-label-char-list-for-kigou-mode)
-             (length tutcode-heading-label-char-list-for-prediction))))
-      (begin
-        (custom-set-value! 'tutcode-nr-candidate-max 10)
-        (custom-set-value! 'tutcode-nr-candidate-max-for-kigou-mode 10)
-        (custom-set-value! 'tutcode-nr-candidate-max-for-prediction 10)
-        (custom-set-value! 'tutcode-nr-candidate-max-for-guide 10)))))
+  tutcode-custom-adjust-nr-candidate-max)
+
+(custom-add-hook 'tutcode-use-pseudo-table-style?
+  'custom-set-hooks
+  tutcode-custom-adjust-nr-candidate-max)
 
 (custom-add-hook 'tutcode-candidate-window-table-layout
 		 'custom-activity-hooks
 		 (lambda ()
-		   tutcode-use-table-style-candidate-window?))
+                   (eq? candidate-window-style 'table)))
 
 (custom-add-hook 'tutcode-bushu-index2-filename
 		 'custom-activity-hooks
 		 (lambda ()
-		   tutcode-use-interactive-bushu-conversion?))
+                   (or
+                     tutcode-use-interactive-bushu-conversion?
+                     (eq? tutcode-bushu-conversion-algorithm 'tc-2.3.1-22.6))))
 
 (custom-add-hook 'tutcode-bushu-expand-filename
 		 'custom-activity-hooks
 		 (lambda ()
-		   tutcode-use-interactive-bushu-conversion?))
+                   (or
+                     tutcode-use-interactive-bushu-conversion?
+                     (eq? tutcode-bushu-conversion-algorithm 'tc-2.3.1-22.6))))
