@@ -89,7 +89,7 @@
     ((("r" "l" "h"))("" ""))
     ((("r" "l" "j"))("" ""))
     ((("e" "l" "k"))("" ""))
-    ((("e" "l" "e" "k"))("" ""))
+    ((("e" "l" "e" "k"))(""))
     ((("e" "l" "i"))("" ""))
     ((("e" "l" "u"))("" ""))
     ((("e" "l" "h"))("" ""))
@@ -121,94 +121,6 @@
     ((("q" "l" "u"))("" ""))
     ((("q" "l" "j"))("" ""))
     ((("a" "l" "k"))("" ""))
-    ((("R" "K"))(""))
-    ((("R" "I"))(""))
-    ((("R" "U"))(""))
-    ((("R" "H"))(""))
-    ((("R" "J"))(""))
-    ((("E" "K"))(""))
-    ((("E" "I"))(""))
-    ((("E" "U"))(""))
-    ((("E" "H"))(""))
-    ((("E" "J"))(""))
-    ((("S" "K"))(""))
-    ((("S" "I"))(""))
-    ((("S" "U"))(""))
-    ((("S" "H"))(""))
-    ((("S" "J"))(""))
-    ((("D" "K"))(""))
-    ((("D" "I"))(""))
-    ((("D" "U"))(""))
-    ((("D" "H"))(""))
-    ((("D" "J"))(""))
-    ((("F" "K"))(""))
-    ((("F" "I"))(""))
-    ((("F" "U"))(""))
-    ((("F" "H"))(""))
-    ((("F" "J"))(""))
-    ((("T" "K"))(""))
-    ((("T" "I"))(""))
-    ((("T" "U"))(""))
-    ((("T" "H"))(""))
-    ((("T" "J"))(""))
-    ((("W" "K"))(""))
-    ((("W" "I"))(""))
-    ((("W" "U"))(""))
-    ((("W" "H"))(""))
-    ((("W" "J"))(""))
-    ((("Q" "K"))(""))
-    ((("Q" "U"))(""))
-    ((("Q" "J"))(""))
-    ((("G" "K"))(""))
-    ((("G" "I"))(""))
-    ((("G" "U"))(""))
-    ((("G" "H"))(""))
-    ((("G" "J"))(""))
-    ((("A" "K"))(""))
-    ((("A" "I"))(""))
-    ((("A" "H"))(""))
-    ((("A" "J"))(""))
-    ((("R" "L" "K"))(""))
-    ((("R" "L" "I"))(""))
-    ((("R" "L" "U"))(""))
-    ((("R" "L" "R" "U"))(""))
-    ((("R" "L" "H"))(""))
-    ((("R" "L" "J"))(""))
-    ((("E" "L" "K"))(""))
-    ((("E" "L" "E" "K"))(""))
-    ((("E" "L" "I"))(""))
-    ((("E" "L" "U"))(""))
-    ((("E" "L" "H"))(""))
-    ((("E" "L" "E" "H"))(""))
-    ((("E" "L" "J"))(""))
-    ((("S" "L" "K"))(""))
-    ((("S" "L" "I"))(""))
-    ((("S" "L" "U"))(""))
-    ((("S" "L" "H"))(""))
-    ((("S" "L" "J"))(""))
-    ((("D" "L" "K"))(""))
-    ((("D" "L" "I"))(""))
-    ((("D" "L" "U"))(""))
-    ((("D" "L" "D" "U"))(""))
-    ((("D" "L" "H"))(""))
-    ((("D" "L" "J"))(""))
-    ((("F" "L"))(""))
-    ((("T" "L" "K"))(""))
-    ((("T" "L" "I"))(""))
-    ((("T" "L" "U"))(""))
-    ((("T" "L" "H"))(""))
-    ((("T" "L" "J"))(""))
-    ((("T" "L" "T" "K"))(""))
-    ((("T" "L" "T" "I"))(""))
-    ((("T" "L" "T" "U"))(""))
-    ((("T" "L" "T" "H"))(""))
-    ((("T" "L" "T" "J"))(""))
-    ((("Q" "L" "K"))(""))
-    ((("Q" "L" "U"))(""))
-    ((("Q" "L" "J"))(""))
-    ((("A" "L" "K"))(""))
-    ((("E" " "))(""))
-    ((("O" " "))(""))
     ((("q" " "))(""))
     ((("w" " "))(""))
     ((("e" " "))(""))
@@ -2764,3 +2676,207 @@
     ((("z" "v" "p"))(""))
     ((("z" "v" "u"))(""))
     ((("z" "v" "y"))(""))))
+
+;;; ѴбΤɲ52ʸȡ
+;;; ʹ񡢶Ʊ̿īʹNHKʤɤȼ˻Ѥ᤿
+;;; бΤɲ11ʸ־ʱ
+;;; http://www.geocities.jp/ken1noguchi/TUT-CODE/kankyo/download.htm
+(define tutcode-rule-shin-joyo-kanji-plus
+  '(
+    ((("d" "g" "g"))(""))
+    ((("p" "h" "h"))(""))
+    ((("q" "t" "t"))(""))
+    ((("j" "n" "n"))(""))
+    ((("k" "h" "h"))(""))
+    ((("f" "g" "g"))(""))
+    ((("q" "g" "g"))(""))
+    ((("." "h" "h"))("½"))
+    ((("e" "v" "v"))(""))
+    ((("e" "g" "g"))("ݵ"))
+    ((("j" "h" "h"))(""))
+    ((("c" "g" "g"))(""))
+    ((("i" "m" "m"))(""))
+    ((("i" "h" "h"))("ª"))
+    ((("," "h" "h"))(""))
+    ((("s" "b" "b"))("Ϩ"))
+    ((("a" "b" "b"))("Ľ"))
+    ((("l" "h" "h"))(""))
+    (((";" "h" "h"))(""))
+    ((("f" "t" "t"))("Ǳ"))
+    ((("," "y" "y"))(""))
+    ((("/" "y" "y"))(""))
+    (((";" "n" "n"))(""))
+    ((("s" "g" "g"))(""))
+    ((("l" "y" "y"))("Ч"))
+    ((("j" "y" "y"))(""))
+    ((("/" "h" "h"))(""))
+    ((("o" "n" "n"))("Ŷ"))
+    ((("d" "t" "t"))(""))
+    ((("s" "t" "t"))("η"))
+    ((("a" "t" "t"))(""))
+    ((("z" "g" "g"))(""))
+    ((("e" "b" "b"))(""))
+    ((("w" "b" "b"))("ܴ"))
+    ((("k" "y" "y"))(""))
+    ((("r" "g" "g"))(""))
+    (((";" "y" "y"))(""))
+    ((("." "y" "y"))(""))
+    ((("i" "n" "n"))("ɲ"))
+    ((("f" "v" "v"))(""))
+    ((("e" "t" "t"))(""))
+    ((("u" "h" "h"))(""))
+    ((("x" "t" "t"))(""))
+    ((("l" "m" "m"))(""))
+    ((("i" "y" "y"))(""))
+    ((("o" "y" "y"))(""))
+    ((("u" "y" "y"))(""))
+    ((("s" "v" "v"))("Ӯ"))
+    ((("o" "h" "h"))("ۣ"))
+    ((("p" "y" "y"))(""))
+    ((("k" "n" "n"))(""))
+    ((("f" "b" "b"))(""))
+
+    ((("l" "n" "n"))(""))
+    ((("d" "v" "v"))(""))
+    ((("d" "b" "b"))(""))
+    ((("r" "t" "t"))(""))
+    ((("a" "g" "g"))(""))
+    ((("w" "g" "g"))(""))
+    ((("z" "t" "t"))(""))
+    ((("c" "t" "t"))(""))
+    ((("w" "t" "t"))(""))
+    ((("k" "m" "m"))("ʱ"))
+    ((("x" "g" "g"))(""))))
+
+(if (and (symbol-bound? 'tutcode-rule-use-tutplus?)
+         tutcode-rule-use-tutplus?)
+  (set! tutcode-rule (append tutcode-rule tutcode-rule-shin-joyo-kanji-plus)))
+
+(define tutcode-rule-uppercase-for-katakana
+  '(
+    ((("R" "K"))("" ""))
+    ((("R" "I"))("" ""))
+    ((("R" "U"))("" ""))
+    ((("R" "H"))("" ""))
+    ((("R" "J"))("" ""))
+    ((("E" "K"))("" ""))
+    ((("E" "I"))("" ""))
+    ((("E" "U"))("" ""))
+    ((("E" "H"))("" ""))
+    ((("E" "J"))("" ""))
+    ((("S" "K"))("" ""))
+    ((("S" "I"))("" ""))
+    ((("S" "U"))("" ""))
+    ((("S" "H"))("" ""))
+    ((("S" "J"))("" ""))
+    ((("D" "K"))("" ""))
+    ((("D" "I"))("" ""))
+    ((("D" "U"))("" ""))
+    ((("D" "H"))("" ""))
+    ((("D" "J"))("" ""))
+    ((("F" "K"))("" ""))
+    ((("F" "I"))("" ""))
+    ((("F" "U"))("" ""))
+    ((("F" "H"))("" ""))
+    ((("F" "J"))("" ""))
+    ((("T" "K"))("" ""))
+    ((("T" "I"))("" ""))
+    ((("T" "U"))("" ""))
+    ((("T" "H"))("" ""))
+    ((("T" "J"))("" ""))
+    ((("W" "K"))("" ""))
+    ((("W" "I"))("" ""))
+    ((("W" "U"))("" ""))
+    ((("W" "H"))("" ""))
+    ((("W" "J"))("" ""))
+    ((("Q" "K"))("" ""))
+    ((("Q" "U"))("" ""))
+    ((("Q" "J"))("" ""))
+    ((("G" "K"))("" ""))
+    ((("G" "I"))("" ""))
+    ((("G" "U"))("" ""))
+    ((("G" "H"))("" ""))
+    ((("G" "J"))("" ""))
+    ((("A" "K"))("" ""))
+    ((("A" "I"))("" ""))
+    ((("A" "H"))("" ""))
+    ((("A" "J"))("" ""))
+    ((("R" "L" "K"))("" ""))
+    ((("R" "L" "I"))("" ""))
+    ((("R" "L" "U"))("" ""))
+    ((("R" "L" "R" "U"))(""))
+    ((("R" "L" "H"))("" ""))
+    ((("R" "L" "J"))("" ""))
+    ((("E" "L" "K"))("" ""))
+    ((("E" "L" "E" "K"))(""))
+    ((("E" "L" "I"))("" ""))
+    ((("E" "L" "U"))("" ""))
+    ((("E" "L" "H"))("" ""))
+    ((("E" "L" "E" "H"))(""))
+    ((("E" "L" "J"))("" ""))
+    ((("S" "L" "K"))("" ""))
+    ((("S" "L" "I"))("" ""))
+    ((("S" "L" "U"))("" ""))
+    ((("S" "L" "H"))("" ""))
+    ((("S" "L" "J"))("" ""))
+    ((("D" "L" "K"))("" ""))
+    ((("D" "L" "I"))("" ""))
+    ((("D" "L" "U"))("" ""))
+    ((("D" "L" "D" "U"))("" ""))
+    ((("D" "L" "H"))("" ""))
+    ((("D" "L" "J"))("" ""))
+    ((("F" "L"))("" ""))
+    ((("T" "L" "K"))("" ""))
+    ((("T" "L" "I"))("" ""))
+    ((("T" "L" "U"))("" ""))
+    ((("T" "L" "H"))("" ""))
+    ((("T" "L" "J"))("" ""))
+    ((("T" "L" "T" "K"))("" ""))
+    ((("T" "L" "T" "I"))("" ""))
+    ((("T" "L" "T" "U"))("" ""))
+    ((("T" "L" "T" "H"))("" ""))
+    ((("T" "L" "T" "J"))("" ""))
+    ((("Q" "L" "K"))("" ""))
+    ((("Q" "L" "U"))("" ""))
+    ((("Q" "L" "J"))("" ""))
+    ((("A" "L" "K"))("" ""))))
+
+;;; ʸǡߤΤҤ餬/ʥ⡼ɤȿФΥ/Ҥ餬ʤϤ
+;;; ˤϡ~/.uim˰ʲΤ褦˵Ҥ롣
+;;;   (define tutcode-rule-uppercase-as-opposite-kana? #t)
+;;; (褫ưϡʸǤɬ)
+(if (or (not (symbol-bound? 'tutcode-rule-uppercase-as-opposite-kana?))
+        (not tutcode-rule-uppercase-as-opposite-kana?))
+  (set! tutcode-rule-uppercase-for-katakana
+    (map
+      (lambda (x)
+        (list (car x) (list (caadr x))))
+      tutcode-rule-uppercase-for-katakana)))
+
+(define tutcode-rule-uppercase-for-kigou-in-katakana
+  '(
+    ((("E" " "))(""))
+    ((("O" " "))(""))))
+
+;;; ʸǥϤԤϿʤ褦ˤˤϡ
+;;; ~/.uim˰ʲΤ褦˵Ҥ롣(ʸǤΥϤȤʤ硢
+;;; stroke-helpβ۸פǲȾʬΥեȥΰ̵ȾʬΥ
+;;; ɽʤ)
+;;;   (define tutcode-rule-exclude-uppercase-for-katakana? #t)
+;;; ޤ˽и뤳Ȥ¿""""ʸϤΤߤ
+;;; Ͽʤ褦ˤˤϡ~/.uim˰ʲΤ褦˵Ҥ롣
+;;;   (define tutcode-rule-exclude-uppercase-for-kigou-in-katakana? #t)
+;;; (XXX:ϡʸ((("E" " "))(""))ȤƤ⡢
+;;;  ϥѴȾʸ롣򤷤ѡ
+;;;  :"CODE "Ǹ"COD"ɽϥѴ"CODe ")
+(if (or (not (symbol-bound? 'tutcode-rule-exclude-uppercase-for-katakana?))
+        (not tutcode-rule-exclude-uppercase-for-katakana?))
+  (set! tutcode-rule
+    (append
+      tutcode-rule
+      tutcode-rule-uppercase-for-katakana
+      (if (or (not (symbol-bound?
+                    'tutcode-rule-exclude-uppercase-for-kigou-in-katakana?))
+              (not tutcode-rule-exclude-uppercase-for-kigou-in-katakana?))
+        tutcode-rule-uppercase-for-kigou-in-katakana
+        ()))))
