@@ -206,6 +206,13 @@ static void uim_custom_key_free(struct uim_custom_key *custom_key);
 static char *extract_key_literal(const struct uim_custom_key *custom_key);
 static char *key_list_to_str(const struct uim_custom_key *const *list, const char *sep);
 
+static char ***uim_custom_table_get(const char *custom_sym, const char *getter_proc);
+static char *literalized_strdup(const char *str);
+static char *row_list_to_str(const char *const *list);
+static char *table_to_str(const char** const *list, const char *sep);
+static struct uim_custom_choice **uim_custom_table_header_item_list(const char *custom_sym);
+
+
 static union uim_custom_value *uim_custom_value_internal(const char *custom_sym, const char *getter_proc);
 static union uim_custom_value *uim_custom_value(const char *custom_sym);
 static union uim_custom_value *uim_custom_default_value(const char *custom_sym);
@@ -439,6 +446,8 @@ uim_custom_type(const char *custom_sym)
     return UCustom_OrderedList;
   } else if (uim_custom_type_eq(custom_sym, "key")) {
     return UCustom_Key;
+  } else if (uim_custom_type_eq(custom_sym, "table")) {
+    return UCustom_Table;
   } else {
     return UCustom_Bool;
   }
@@ -793,6 +802,74 @@ uim_custom_key_list_free(struct uim_custom_key **list)
 		      (uim_scm_c_list_free_func)uim_custom_key_free);
 }
 
+/* table */
+static char ***
+uim_custom_table_get(const char *custom_sym, const char *getter_proc)
+{
+  char ***custom_table;
+  int row_count;
+  int row;
+
+  UIM_EVAL_FSTRING1(NULL, "(length %s)", custom_sym);
+  row_count = uim_scm_c_int(uim_scm_return_value());
+
+  custom_table = (char ***)malloc(sizeof(char **) * (row_count + 1));
+  if (!custom_table)
+    return NULL;
+
+  custom_table[row_count] = NULL;
+  for (row = 0; row < row_count; row++) {
+    int column_count;
+    int column;
+    UIM_EVAL_FSTRING2(NULL, "(length (nth %d %s))", row, custom_sym);
+    column_count = uim_scm_c_int(uim_scm_return_value());
+
+    custom_table[row] = (char **)malloc(sizeof(char *) * (column_count + 1));
+    if (!custom_table[row])
+      return NULL;
+    custom_table[row][column_count] = NULL;
+    for (column = 0; column < column_count; column++) {
+      char *str;
+      UIM_EVAL_FSTRING3(NULL, "(nth %d (nth %d %s))", column, row, custom_sym);
+      str = uim_scm_c_str(uim_scm_return_value());
+      if (!str)
+        return NULL;
+      custom_table[row][column] = malloc(sizeof(char) * (strlen(str) + 1));
+      if (!custom_table[row][column])
+        return NULL;
+      custom_table[row][column] = str;
+    }
+  }
+
+  return custom_table;
+}
+
+static char *
+literalized_strdup(const char *str)
+{
+  return strdup(literalize_string(str));
+}
+
+static char *
+row_list_to_str(const char *const *list)
+{
+  return c_list_to_str((const void *const *)list,
+                       (char *(*)(const void *))literalized_strdup, " ");
+}
+
+static char *
+table_to_str(const char** const *list, const char *sep)
+{
+  return c_list_to_str((const void *const *)list,
+		       (char *(*)(const void *))row_list_to_str, sep);
+}
+
+static struct uim_custom_choice **
+uim_custom_table_header_item_list(const char *custom_sym)
+{
+  return uim_custom_choice_item_list(custom_sym);
+}
+
 static union uim_custom_value *
 uim_custom_value_internal(const char *custom_sym, const char *getter_proc)
 {
@@ -834,6 +911,9 @@ uim_custom_value_internal(const char *custom_sym, const char *getter_proc)
   case UCustom_Key:
     value->as_key = uim_custom_key_get(custom_sym, getter_proc);
     break;
+  case UCustom_Table:
+    value->as_table = uim_custom_table_get(custom_sym, getter_proc);
+    break;
   default:
     free(value);
     value = NULL;
@@ -921,6 +1001,10 @@ uim_custom_range_get(const char *custom_sym)
   case UCustom_OrderedList:
     range->as_olist.valid_items = uim_custom_olist_item_list(custom_sym);
     break;
+  case UCustom_Table:
+    range->as_table_header.valid_items
+        = uim_custom_table_header_item_list(custom_sym);
+    break;
   }
 
   return range;
@@ -1469,6 +1553,15 @@ uim_custom_set(const struct uim_custom *custom)
       free(val);
     }
     break;
+  case UCustom_Table:
+    {
+      char *val
+        = table_to_str((const char ** const *)custom->value->as_table, ") (");
+      UIM_EVAL_FSTRING2(NULL, "(custom-set-value! '%s '((%s)))",
+        custom->symbol, val);
+      free(val);
+    }
+    break;
   default:
     return UIM_FALSE;
   }
