@@ -287,6 +287,21 @@ im_activate_candidate_selector(uim_lisp uc_,
   return uim_scm_f();
 }
 
+static uim_lisp
+im_delay_activate_candidate_selector(uim_lisp uc_, uim_lisp delay_)
+{
+  uim_context uc;
+  int delay;
+
+  uc = retrieve_uim_context(uc_);
+  delay = C_INT(delay_);
+
+  if (uc->candidate_selector_delay_activate_cb)
+    uc->candidate_selector_delay_activate_cb(uc->ptr, delay);
+
+  return uim_scm_f();
+}
+
 static uim_lisp
 im_select_candidate(uim_lisp uc_, uim_lisp idx_)
 {
@@ -332,6 +347,18 @@ im_deactivate_candidate_selector(uim_lisp uc_)
   return uim_scm_f();
 }
 
+static uim_lisp
+im_delay_activate_candidate_selector_supportedp(uim_lisp uc_)
+{
+  uim_context uc;
+
+  uc = retrieve_uim_context(uc_);
+
+  if (uc->candidate_selector_delay_activate_cb)
+    return uim_scm_t();
+  return uim_scm_f();
+}
+
 static uim_lisp
 im_acquire_text(uim_lisp uc_, uim_lisp text_id_, uim_lisp origin_,
 		uim_lisp former_len_, uim_lisp latter_len_)
@@ -460,6 +487,11 @@ uim_init_im_subrs(void)
   uim_scm_init_proc1("im-deactivate-candidate-selector",
 		     im_deactivate_candidate_selector);
 
+  uim_scm_init_proc2("im-delay-activate-candidate-selector",
+		     im_delay_activate_candidate_selector);
+  uim_scm_init_proc1("im-delay-activate-candidate-selector-supported?",
+		     im_delay_activate_candidate_selector_supportedp);
+
   uim_scm_init_proc5("im-acquire-text-internal", im_acquire_text);
   uim_scm_init_proc5("im-delete-text-internal", im_delete_text);
 
