@@ -105,8 +105,9 @@ init_server_fd(char *path)
   logname = getenv("LOGNAME");
   if (logname) {
     pw = getpwnam(logname);
-    if (pw)
+    if (pw) {
       fchown(fd, pw->pw_uid, -1);
+    }
   }
 
   if ((flag = fcntl(fd, F_GETFL)) < 0) {
