@@ -37,13 +37,12 @@
 
 #include <stdlib.h>
 
+#include "version.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#define UIM_VERSION_MAJOR      (1)
-#define UIM_VERSION_MINOR      (7)
-#define UIM_VERSION_PATCHLEVEL (2)
 #define UIM_VERSION_REQUIRE(major, minor, patchlevel)			     \
   ((major) < UIM_VERSION_MAJOR						     \
    || ((major) == UIM_VERSION_MAJOR && (minor) < UIM_VERSION_MINOR)	     \
@@ -688,6 +687,31 @@ void uim_set_candidate_selector_cb(uim_context uc,
                                                          int direction),
                                    void (*deactivate_cb)(void *ptr));
 
+/**
+ * Set callback function to support delay showing candidate-selection.
+ *
+ * @param uc input context
+ * @param delay_activate_cb called when candidate window should be activated with delay.
+ *
+ * @see uim_create_context
+ */
+void uim_set_delay_candidate_selector_cb(uim_context uc,
+                                         void (*delay_activate_cb)(void *ptr,
+                                                                   int delay));
+
+/**
+ * Notify that the candidate selector is being activated after delay.
+ *
+ * The input context must update number of candidates,
+ * display limit and selected index.
+ *
+ * @param uc input context
+ * @param nr [out] total number of candidates
+ * @param display_limit [out] number of candidates to show on one page
+ * @param selected_index [out] index of selected candidate
+ */
+void uim_delay_activating(uim_context uc, int *nr, int *display_limit, int *selected_index);
+
 /**
  * Get candidate data.
  *
@@ -827,7 +851,7 @@ uim_set_mode_list_update_cb(uim_context uc,
  * completely, adding separated surrounding text deletion interface is better
  * solution at now.  -- YamaKen 2006-10-07
  *
- * http://uim.googlecode.com/svn/branches/composer/scm/event.scm
+ * http://uim.googlecode.com/git-history/composer/scm/event.scm
  *
  * (define-event 'commit
  *   upward-event-rec-spec
