@@ -692,8 +692,11 @@ void XimIM::create_compose_tree()
 	}
     }
 
-    if (name[0] == '\0' && !get_compose_filename(name, sizeof(name)))
+    if (name[0] == '\0' && !get_compose_filename(name, sizeof(name))) {
+        if (fp)
+            fclose(fp);
 	return;
+    }
 
     if (fp == NULL && ((fp = fopen(name, "r")) == NULL))
 	return;
