@@ -43,6 +43,7 @@
 #include <cstdlib>
 #include <cstring>
 #include <sys/select.h>
+#include <time.h>
 
 #include "xim.h"
 #include "xdispatch.h"
@@ -58,7 +59,6 @@
 
 Display *XimServer::gDpy;
 std::map<Window, XimServer *> XimServer::gServerMap;
-CandWinPosType XimServer::gCandWinPosType;
 
 // Configuration
 int g_option_mask;
@@ -93,6 +93,13 @@ static char *supported_locales;
 std::list<UIMInfo> uim_info;
 static void check_pending_xevent(void);
 
+#if UIM_XIM_USE_DELAY
+static void timer_check(void);
+static void *timer_ptr;
+static void (*timer_cb)(void *ptr);
+static time_t timer_time;
+#endif
+
 bool
 pretrans_register()
 {
@@ -144,7 +151,11 @@ static void main_loop()
     while (1) {
 	FD_ZERO(&rfds);
 	FD_ZERO(&wfds);
+#if UIM_XIM_USE_DELAY
+	tv.tv_sec = 1;
+#else
 	tv.tv_sec = 2;
+#endif
 	tv.tv_usec = 0;
 
 	std::map<int, fd_watch_struct>::iterator it;
@@ -160,6 +171,9 @@ static void main_loop()
 	}
 	if ((select(fd_max + 1, &rfds, &wfds, NULL, &tv)) == 0) {
 	    check_pending_xevent();
+#if UIM_XIM_USE_DELAY
+	    timer_check();
+#endif
 	    continue;
 	}
 
@@ -179,6 +193,9 @@ static void main_loop()
 		break;
 	    ++it;
 	}
+#if UIM_XIM_USE_DELAY
+	timer_check();
+#endif
     }
 }
 
@@ -339,6 +356,31 @@ xEventRead(int /* fd */, int /* ev */)
     check_pending_xevent();
 }
 
+#if UIM_XIM_USE_DELAY
+static void
+timer_check(void)
+{
+    if (timer_time > 0 && time(NULL) >= timer_time) {
+	timer_time = 0;
+	timer_cb(timer_ptr);
+    }
+}
+
+void
+timer_set(int seconds, void (*timeout_cb)(void *ptr), void *ptr)
+{
+    timer_time = time(NULL) + seconds;
+    timer_cb = timeout_cb;
+    timer_ptr = ptr;
+}
+
+void
+timer_cancel()
+{
+    timer_time = 0;
+}
+#endif
+
 static void
 error_handler_setup()
 {
@@ -428,7 +470,10 @@ init_supported_locales()
     const char *s;
     int len;
 
-    asprintf(&supported_locales, "@locale=");
+    if (asprintf(&supported_locales, "@locale=") == -1) {
+        free(supported_locales);
+        return;
+    }
     len = static_cast<int>(strlen(supported_locales));
 
     // get all locales
@@ -642,6 +687,7 @@ main(int argc, char **argv)
     if (uim_scm_symbol_value_bool("uim-xim-use-xft-font?"))
 	init_default_xftfont(); // setup Xft fonts for Ov/Rw preedit
 #endif
+    check_candwin_style();
     check_candwin_pos_type();
 
     // Handle pending events to prevent hang just after startup
