@@ -42,6 +42,7 @@
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <X11/Xutil.h>
+#include <X11/XKBlib.h>
 
 /* workaround for pre X11R6.7 */
 #ifndef XK_KOREAN
@@ -84,6 +85,9 @@ static int check_modifier(std::list<KeySym> list);
 static int gMod1Mask, gMod2Mask, gMod3Mask, gMod4Mask, gMod5Mask;
 static int gXNumLockMask;
 
+CandWinPosType XimServer::gCandWinPosType;
+CandWinStyle XimServer::gCandWinStyle;
+bool XimServer::gCandWinStyleUpdated;
 
 void print_ustring(uString *s)
 {
@@ -175,6 +179,9 @@ void XimServer::customContext(const char *custom, const char *val) {
 
     if (!strcmp(custom, "candidate-window-position"))
 	check_candwin_pos_type();
+
+    if (!strcmp(custom, "candidate-window-style"))
+	check_candwin_style();
 }
 
 void XimServer::reloadConfigs() {
@@ -200,6 +207,7 @@ void XimServer::reloadConfigs() {
 	disp->hide_caret_state();
     }
 
+    check_candwin_style();
     check_candwin_pos_type();
 }
 
@@ -342,6 +350,9 @@ InputContext::InputContext(XimServer *svr, XimIC *ic, const char *engine)
 
 InputContext::~InputContext()
 {
+#if UIM_XIM_USE_DELAY
+    timer_cancel();
+#endif
     if (mFocusedContext == this)
 	mFocusedContext = NULL;
 
@@ -431,6 +442,10 @@ InputContext::createUimContext(const char *engine)
 	uim_set_im_switch_request_cb(uc,
 			InputContext::switch_app_global_im_cb,
 			InputContext::switch_system_global_im_cb);
+#if UIM_XIM_USE_DELAY
+	uim_set_delay_candidate_selector_cb(uc,
+			InputContext::candidate_activate_with_delay_cb);
+#endif
 
 	if (mFocusedContext == this)
 	    uim_prop_list_update(uc);
@@ -491,7 +506,10 @@ void InputContext::switch_system_global_im(const char *name)
     for (it = XimServer::gServerMap.begin(); it != XimServer::gServerMap.end(); ++it)
 	(*it).second->changeContext(name);
 
-    asprintf(&msg, "im_change_whole_desktop\n%s\n", name);
+    if (asprintf(&msg, "im_change_whole_desktop\n%s\n", name) == -1) {
+        free(msg);
+        return;
+    }
     uim_helper_send_message(lib_uim_fd, msg);
     free(msg);
 }
@@ -628,6 +646,20 @@ void InputContext::candidate_activate_cb(void *ptr, int nr, int display_limit)
     ic->candidate_activate(nr, display_limit);
 }
 
+#if UIM_XIM_USE_DELAY
+void InputContext::candidate_activate_with_delay_cb(void *ptr, int delay)
+{
+    InputContext *ic = (InputContext *)ptr;
+    ic->candidate_activate_with_delay(delay);
+}
+
+void InputContext::candidate_activate_timeout_cb(void *ptr)
+{
+    InputContext *ic = (InputContext *)ptr;
+    ic->candidate_activate_timeout();
+}
+#endif
+
 void InputContext::candidate_select_cb(void *ptr, int index)
 {
     InputContext *ic = (InputContext *)ptr;
@@ -815,10 +847,11 @@ void InputContext::candidate_activate(int nr, int display_limit)
     std::vector<const char *> candidates;
     std::vector<const char *>::iterator it;
 
+#if UIM_XIM_USE_DELAY
+    timer_cancel();
+#endif
     Canddisp *disp = canddisp_singleton();
 
-    if (nr > display_limit)
-        display_limit = disp->adjust_display_limit(mUc, display_limit);
     mDisplayLimit = display_limit;
     if (display_limit)
 	mNumPage = (nr - 1) / display_limit + 1;
@@ -880,6 +913,30 @@ void InputContext::candidate_activate(int nr, int display_limit)
     need_hilite_selected_cand = false;
 }
 
+#if UIM_XIM_USE_DELAY
+void InputContext::candidate_activate_with_delay(int delay)
+{
+    timer_cancel();
+    if (delay > 0) {
+	timer_set(delay, InputContext::candidate_activate_timeout_cb, this);
+    } else {
+	candidate_activate_timeout();
+    }
+}
+
+void InputContext::candidate_activate_timeout()
+{
+    int nr = -1, display_limit = -1, selected_index = -1;
+    uim_delay_activating(mUc, &nr, &display_limit, &selected_index);
+    if (nr > 0) {
+	candidate_activate(nr, display_limit);
+	if (selected_index >= 0) {
+	    candidate_select(selected_index);
+	}
+    }
+}
+#endif
+
 void InputContext::candidate_update()
 {
     Canddisp *disp = canddisp_singleton();
@@ -1014,6 +1071,9 @@ void InputContext::candidate_shift_page(int direction)
 
 void InputContext::candidate_deactivate()
 {
+#if UIM_XIM_USE_DELAY
+    timer_cancel();
+#endif
     if (mCandwinActive) {
 	std::vector<const char *>::iterator i;
 	Canddisp *disp = canddisp_singleton();
@@ -1087,9 +1147,10 @@ void InputContext::update_prop_list(const char *str)
 {
     char *buf;
 
-    asprintf(&buf, "prop_list_update\ncharset=UTF-8\n%s", str);
-    if (!buf)
-	return;
+    if (asprintf(&buf, "prop_list_update\ncharset=UTF-8\n%s", str) == -1) {
+        free(buf);
+        return;
+    }
     uim_helper_send_message(lib_uim_fd, buf);
     free(buf);
 
@@ -1131,9 +1192,10 @@ void InputContext::update_prop_label(const char *str)
 {
     char *buf;
 
-    asprintf(&buf, "prop_label_update\ncharset=UTF-8\n%s", str);
-    if (!buf)
-	return;
+    if (asprintf(&buf, "prop_label_update\ncharset=UTF-8\n%s", str) == -1) {
+        free(buf);
+        return;
+    }
     uim_helper_send_message(lib_uim_fd, buf);
     free(buf);
 #if 0    
@@ -1410,8 +1472,8 @@ void init_modifier_keys() {
 		KeySym ks;
 		int index = 0;
 		do {
-		    ks = XKeycodeToKeysym(XimServer::gDpy,
-				    map->modifiermap[k], index);
+		    ks = XkbKeycodeToKeysym(XimServer::gDpy,
+				    map->modifiermap[k], 0, index);
 		    index++;
 		} while (!ks && index < keysyms_per_keycode);
 
@@ -1448,6 +1510,23 @@ void init_modifier_keys() {
 #endif
 }
 
+void
+check_candwin_style()
+{
+    char *style = uim_scm_symbol_value_str("candidate-window-style");
+    CandWinStyle PrevStyle = XimServer::gCandWinStyle;
+
+    if (style && !strcmp(style, "table"))
+	XimServer::gCandWinStyle = Table;
+    else if (style && !strcmp(style, "horizontal"))
+	XimServer::gCandWinStyle = Horizontal;
+    else
+	XimServer::gCandWinStyle = Vertical;
+
+    XimServer::gCandWinStyleUpdated =
+	    (PrevStyle != XimServer::gCandWinStyle) ? true : false;
+    free(style);
+}
 
 void
 check_candwin_pos_type()
