@@ -45,6 +45,7 @@
 #include "compose.h"
 
 #define UIM_XIM_USE_NEW_PAGE_HANDLING 1
+#define UIM_XIM_USE_DELAY 1
 
 // preedit ornament
 #define PE_NORMAL 0
@@ -67,6 +68,12 @@ typedef enum {
     Right
 } CandWinPosType;
 
+typedef enum {
+    Vertical,
+    Horizontal,
+    Table 
+} CandWinStyle;
+
 // state of preedit.
 // created in the constructor of InputContext, and deleted in the
 // destructor of it.
@@ -94,7 +101,12 @@ void init_default_xftfont();
 void update_default_xftfont();
 #endif
 void reload_uim(int x);
+void check_candwin_style();
 void check_candwin_pos_type();
+#if UIM_XIM_USE_DELAY
+void timer_set(int seconds, void (*timeout_cb)(void *ptr), void *ptr);
+void timer_cancel();
+#endif
 
 
 // for command line option
@@ -169,6 +181,10 @@ public:
     void pushback_preedit_string(int attr, const char *str);
     void update_preedit();
     void candidate_activate(int nr, int display_limit);
+#if UIM_XIM_USE_DELAY
+    void candidate_activate_with_delay(int delay);
+    void candidate_activate_timeout();
+#endif
     void candidate_select(int index);
     void candidate_shift_page(int direction);
     void candidate_deactivate();
@@ -196,6 +212,10 @@ public:
     static void pushback_cb(void *ptr, int attr, const char *str);
     static void update_cb(void *ptr);
     static void candidate_activate_cb(void *ptr, int nr, int index);
+#if UIM_XIM_USE_DELAY
+    static void candidate_activate_with_delay_cb(void *ptr, int delay);
+    static void candidate_activate_timeout_cb(void *ptr);
+#endif
     static void candidate_select_cb(void *ptr, int index);
     static void candidate_shift_page_cb(void *ptr, int direction);
     static void candidate_deactivate_cb(void *ptr);
@@ -278,6 +298,8 @@ public:
     static Display *gDpy;
     static std::map<Window, XimServer *> gServerMap;
     static CandWinPosType gCandWinPosType;
+    static CandWinStyle gCandWinStyle;
+    static bool gCandWinStyleUpdated;
 private:
     Window mSelectionWin;
     Atom mServerAtom;
